/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.client.internal.WorkItemClient");

dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.client.xhr");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.require("dojox.xml.DomParser");

(function() {

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
var DomParser= dojox.xml.DomParser;

com.ibm.team.build.web.client.internal.WorkItemClient = {	
		
	loadDocument: function(documentURL, onSuccess) {
		// Make an HTTP request on a secured resource.
    	var deferred = jazz.client.xhrGet({
        	url: documentURL,
       		headers: {}
    	});

    	deferred.addCallback(function(result) {
    		onSuccess(result);
        	//return result; // for next in deferred chain
    	});

    	deferred.addErrback(dojo.hitch(this, this._handleloadDocumentError, documentURL));
	},
	
	_handleloadDocumentError: function(documentURL, errorObj) {
   		// This will get called either if the user failed to authenticate and canceled the
    	// dialog or if authentication succeeded but the request had some other problem.
    	if(errorObj.dojoType === "cancel") {
        	alert("User cancelled request.");
    	} else {
        	alert("An error occurred while accessing " +  documentURL + ": "+ errorObj);
    	}
    	return errorObj;
	},
	
	findRemoteItemPickerInfo : function(serviceXML) {
		var path = [ "oslc_cm:ServiceDescriptor", "oslc_cm:changeRequests", "oslc_cm:selectionDialog[@oslc_cm:default='true']"];
		var jsdom = DomParser.parse(serviceXML);
    	var node= this._getNode(jsdom, path);
    	if (node) {
        	return {
        		title: this._getNodeText(node, ["dc:title"]),
            	url: this._getNodeText(node, ["oslc_cm:url"]),
            	widthHint: this._getAttribute(node, "oslc_cm:hintWidth"),
            	heightHint: this._getAttribute(node, "oslc_cm:hintHeight"),
            	namespace: "oslc_cm"
        	};
    	}
    	return null;
	},
	
	// -------------- XML queries -----------------------
	_getNodeText: function(jsdom, segments) {
	    var node= this._getNode(jsdom, segments);
	    if (node) 
	        return this._evalNodeText(node);
	    return null;
	},
	
	NAMESPACES: {
    "oslc_cm": "http://open-services.net/xmlns/cm/1.0/", 
    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#", 
    "calm": "http://jazz.net/xmlns/prod/jazz/calm/1.0/", 
    "dc": "http://purl.org/dc/terms/", 
    "oslc_disc": "http://open-services.net/xmlns/discovery/1.0/",
    "jp": "http://jazz.net/xmlns/prod/jazz/process/1.0/"
	},
	
	_getNode: function(xmlNode, pathSegments) {
	    if (pathSegments.length == 0) {
    	    return xmlNode;
    	}
    
    	var first= pathSegments[0];
    	var segments= pathSegments.slice(1); // make a copy
    
    	var m1= first.match(/([\d\w-_:]+)\[(@?[\d\w-_:]+)='([^']+)'\]/);   // 0[1='2']
    	if (!m1) {
        	// simple element name
        	var nodes= this._getElementByTagName(xmlNode, first);
        	for (var i= 0; i < nodes.length; i++) {
            	var res= this._getNode(nodes[i], segments);
            	if (res)
                	return res;
        	}
        	return null;
    	}
	    var elementName= m1[1];
	    var matchName= m1[2];
	    var matchVal= m1[3];
	    
	    var isAttribMatch= matchName.charAt(0) == '@';
	    if (isAttribMatch)
	       matchName= matchName.substring(1);
	
	
	    var nodes= this._getElementByTagName(xmlNode, elementName);
	    for (var i= 0; i < nodes.length; i++) {
	       var node= nodes[i];
	       if (isAttribMatch) {
	           if (this._getAttribute(node, matchName) == matchVal)
	               return this._getNode(node, segments);
	       } else {
	           var innerNodes= this._getElementByTagName(node, matchName);
	           if (innerNodes.length == 1 && this._evalNodeText(innerNodes[0]) == matchVal)
	               return this._getNode(node, segments);
	       }
	    }
	    return null;
	},
	
	_getElementByTagName: function(node, name) {
	    var s= name.split(':', 2);
	    if (s.length == 1) {
	        return node.getElementsByTagName(name);
	    } else {
	        var ns= this.NAMESPACES[s[0]];
	        return node.getElementsByTagNameNS(s[1], ns);
	    }
	},
	
	_getAttribute: function(node, name) {
	    var s= name.split(':', 2);
	    if (s.length == 1) {
	        return node.getAttribute(name);
	    } else {
	        var ns= this.NAMESPACES[s[0]];
	        return node.getAttributeNS(s[1], ns);
	    }
	},
	
	_evalNodeText: function(node) {
	    if (node.childNodes.length == 1 && node.childNodes[0].nodeName == '#text') {
	         return node.childNodes[0].nodeValue;
	    }
	    return null;
	}
};
	
})();
