/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.MetaNav");

dojo.require("dijit.InlineEditBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("jazz.ui.internal.SimpleToolbar");
dojo.require("jazz.ui.Breadcrumbs");

dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");

dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DraggableTabs = com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTabs;
var HTML = jazz.util.html;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var ResizeNotifier = jazz.util.ResizeNotifier;

var messages;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.MetaNav",
	[dijit._Widget, dijit._Templated, Observable], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/MetaNav.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		dojo.addClass(this._dashboardTitle, "loading-title");
		this._dashboardTitle.innerHTML = messages.loading;
		this._toolbar = new jazz.ui.internal.SimpleToolbar({});
		this._headerLinks.appendChild(this._toolbar.domNode);
		this._breadcrumbs = new jazz.ui.Breadcrumbs({}, this._breadcrumbs);
	},
	init: function(args) {		
		if (this.isInitialized) {
			return;
		}
		
		//set dashboardBundle and setup other useful pointers
		this.dashboardBundle = args.dashboardBundle;
		this.dashboardItem = this.dashboardBundle.dashboard;
		this.dashboardContainer = args.dashboardContainer;
		this.allowSharing = args.allowSharing;
		
		this._dashboardTitle.innerHTML = "";
		dojo.removeClass(this._dashboardTitle, "loading-title");
		
		this._toolbar.addButton(this.dashboardContainer, this.dashboardContainer.refresh, {imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/metanav/icons/refresh.gif"), label: messages.refresh});
		
		this.setEditMode(args.editMode);
		
		//initialize publish API for showing/hiding the message area (info, error, warning)
		dojo.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, this, "_showErrorMessage");
		dojo.subscribe(com.ibm.team.dashboard.web.ui.internal.Constants.hideServerError, this, "hideMessage");
		
		var perms = this.dashboardBundle.permissions;
		if (!this.dashboardContainer.isPrivateContributorDashboard() && (perms.viewTeam || perms.viewPersonal))
			this._addBreadcrumbs(this.dashboardItem);
		else {
			this._breadcrumbs.domNode.style.display = "none";
			this.dashboardContainer._tabWrapper.style.marginTop = "3px";
		}
	
		// show a notice if scope item is archived
		if (this.dashboardBundle.isScopeArchived) {
			var msg;
			var scope = this.dashboardItem.scope;
			switch(scope) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					msg = messages.archivedContributor;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					msg = messages.archivedTeam;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					msg = messages.archivedProject;
					break;
			}
			this.showMessage({type: "info", message: msg});
		}
	},
	_showErrorMessage: function(errorObj) {
		this.dashboardContainer.errorState = true;
		if (!errorObj || errorObj.status == 0 || errorObj.status == 12002 || errorObj.status >= 12029)
			this.showMessage({message: messages.connectionError, actions: [{label: messages.reload, callback: dojo.hitch(this.dashboardContainer,this.dashboardContainer.refresh)}], type: "error"});
		else
			this.showMessage({message: messages.communicationError, actions: [{label: messages.reload, callback: dojo.hitch(this.dashboardContainer,this.dashboardContainer.refresh)}], type: "error"});
	},
	_addBreadcrumbs: function(/*Array*/ breadcrumbs) {
		var breadcrumbs = new Array();
		var defaults = this.dashboardBundle.defaults;
		breadcrumbs.push({label: messages.allHeading, href: Util.getAllDashboardsHref()});
		if (this.dashboardItem.scope == DashboardConstants.SCOPE_TEAM_AREA) {
			breadcrumbs.push({label: defaults.project.name,	href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_PROJECT_AREA, defaults.project.name)});
			var path = defaults.team.path;
			var curPath = [];
			for (var i=0;i<path.length-1;i++) {
				curPath.push(path[i]);
				breadcrumbs.push({
					label: path[i],
					href: DashboardUtil.getDashboardHref(DashboardConstants.SCOPE_TEAM_AREA, curPath)
				});
			}
		}
		
		this._breadcrumbs.segments = breadcrumbs;
		this._breadcrumbs.refresh();
	},
	destroy: function() {
		if (this._title) {
			this._title.destroy();
			this._title = null;
		}
		if (this._windowResizeEvent) {
			ResizeNotifier.unregister(this._windowResizeEvent);
		}
		if (this._refreshEvent)
			dojo.disconnect(this._refreshEvent);
		this.inherited(arguments);
	},	
	setEditMode: function(editMode) {
		var beforeMode = this.editMode;
		this.editMode = editMode;
		if (editMode != beforeMode) {
			if (editMode == Constants.EDIT_AUTO_SAVE || editMode == Constants.EDIT_DEFERRED_SAVE) {
				if (!this._title) {
					//create editable dashboard title
					var node = document.createElement("span");
					node.innerHTML = HTML.escape(this.dashboardItem.title);
					this._dashboardTitle.appendChild(node);
			        this._title = new dijit.InlineEditBox({title:messages.clickToEdit, autoSave: true, width: "400px", noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(this, this._onTitleChange)}, node);
			        this._title.startup();					
				}
				this.showSaveControls(true);
				this._setAutoSave();
			}
			else {
				if (this._title) {
					this._title.destroy();
					this._title = null;
				}
				this._dashboardTitle.innerHTML = HTML.escape(this.dashboardItem.title);
				this.showSaveControls(false);
				if (typeof(this.beforeMode) != "undefined" && this.beforeMode != Constants.READ_ONLY) 
					this.dashboardContainer.setDirty(false);
			}
		}
	},
	showSaveControls: function (showControls){
		if(!this._editEvents) {
			this._editEvents = new Array();
		}
		if (showControls) {
			var autoSave = document.createElement("div");
			autoSave.className = "auto-save";
			var autoSaveInput = document.createElement("input");
			autoSaveInput.type = "checkbox";
			autoSaveInput.id = "autoSave-"+this.dashboardItem._id;
			
			var autoSaveLabel = document.createElement("label");
			autoSaveLabel.htmlFor = "autoSave-"+this.dashboardItem._id;
			autoSaveLabel.innerHTML = messages.autoSave;
			autoSave.appendChild(autoSaveInput);
			autoSave.appendChild(autoSaveLabel);
						
			var save = document.createElement("input");
			save.type = "button";
			save.value = messages.save;
			save.className = "saveButton";
			save.disabled = "disabled";

			this._autoSave = this._toolbar.addWidget({domNode: autoSave, autoSaveInput: autoSaveInput});
			this._saveButton = this._toolbar.addWidget({domNode: save});
			if (typeof this.editMode != "undefined" && this.editMode == Constants.EDIT_AUTO_SAVE) 
				this._toolbar.hideItem(this._saveButton);
			
			this._editEvents.push(this.connect(save,"onclick",this._saveEditingHandler));
			this._editEvents.push(this.connect(autoSaveInput,"onclick",this._setAutoSave));
		} else {
			while(this._editEvents && this._editEvents.length > 0) {
				this.disconnect(this._editEvents.pop());	
			}
			if (this._saveButton) {
				this._toolbar.removeItem(this._saveButton);
			}
			if (this._autoSave) {
				this._toolbar.removeItem(this._autoSave);
			}
			this._saveButton = null;
			this._autoSave = null;
		}
		if (!this._separator && this.dashboardBundle && (this.dashboardBundle.permissions["modify"] || this.dashboardBundle.permissions["modify"]))
			this._separator = this._toolbar.addSeparator(0);
		if (this.allowSharing && !this._shareButton && this.dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle && this.dashboardBundle.permissions["modify"]) {
			this._shareButton = this._toolbar.addButton(this,this._shareDashboard,{imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/metanav/icons/share-dash.gif"), label: messages.sharing, index: 0});
		}
		if (!this._pasteButton && this.dashboardBundle && this.dashboardBundle.permissions["modify"])
			this._pasteButton = this._toolbar.addButton(this,this._paste,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/metanav/icons/paste_edit.gif"), label: messages.paste});
		if (!this._deleteButton && this.dashboardItem.scope == DashboardConstants.SCOPE_CONTRIBUTOR && this.dashboardBundle && this.dashboardBundle.permissions["delete"]) {
			this._deleteButton = this._toolbar.addButton(this,this._deleteDashboard,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/metanav/icons/delete.gif"), label: messages.del});
		}
		if (!this._restoreButton && this.dashboardBundle && this.dashboardBundle.permissions["modify"]) {
			this._restoreButton = this._toolbar.addButton(this,this._restoreDashboard,{index: 0, imgSrc: dojo.moduleUrl("com.ibm.team.dashboard.web","graphics/metanav/icons/restore-default.gif"), label: messages.restoreDefault});
		}
		
	},
	
	_paste: function(event) {
		if (event) 
			dojo.stopEvent(event);
		try {
			var data = globalStorage[location.hostname]["com-ibm-team-dashboard-clipboard"];
		} catch(e) {
			alert(messages.featureNotSupported);
			return;
		}
		if (!data) {
			alert(messages.clipboardEmpty);
			return;
		}
		var data = dojo.fromJson(data);
		if (data.columns) {
			this.dashboardContainer.addPageFromHelper(data);
		} else {
			this.dashboardContainer.currentPage.addViewletFromHelper(data);
		}
	},
	_setAutoSave: function(event) {
		if((event && event.target.checked) || (!event && this.dashboardContainer.autoSave)) {
			if (event) {
				this.dashboardContainer.setEditMode(Constants.EDIT_AUTO_SAVE, true);
				this.editMode = Constants.EDIT_AUTO_SAVE;
			}
			var wasDirty = this.dashboardContainer.isDirty();
			this._toolbar.hideItem(this._saveButton);
			this.dashboardContainer.setDirty(false);
			this.dashboardContainer.autoSave = true;
			this._autoSave.autoSaveInput.checked = "checked";
			if (wasDirty) {
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			}
		} else {
			if (event) {
				this.dashboardContainer.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
				this.editMode = Constants.EDIT_DEFERRED_SAVE;
			}
			this._toolbar.showItem(this._saveButton);
			this.dashboardContainer.autoSave = false;
		}
	},
	_saveEditingHandler: function(event){
		if (event) 
			dojo.stopEvent(event);
		this.dashboardContainer.saveDashboard();
		this.dashboardContainer.setDirty(false);
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", this.dashboardItem.title);
		}
	},
	_shareDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var shared = this.dashboardItem.contextId != this.dashboardItem.scopeItem.itemId;
		if (this.editMode == Constants.EDIT_DEFERRED_SAVE && this.dashboardContainer.isDirty()) {
			alert(messages.alertShareDirty);
			return;
		}
		var selection = (this.dashboardItem.contextId == Constants.CONTEXT_PUBLIC || this.dashboardItem.contextId == this.dashboardItem.scopeItem.itemId) ?
				com.ibm.team.process.web.ui.internal.processAreaSelectionDialog.ALL_PROJECTS_ID : this.dashboardItem.contextId;
		var onOk= dojo.hitch(this,function(attributes) {
			this._onShareOk(attributes);
			this.sharingDialog.close();
		});
		var onCancel= dojo.hitch(this, function() {
			this.sharingDialog.close();
		});
		var contentWidget = new com.ibm.team.dashboard.web.ui.internal.MetaNavSharingDialogContent({
			onOk: onOk,
			onCancel: onCancel,
			projects: true,
			shared: shared,
			selection: selection,
			showAllOption: true
		});
		this.sharingDialog = new jazz.ui.Dialog({primaryTitle: messages.sharing, contentNode: contentWidget.domNode}); 
		contentWidget.init();
	},
	_onShareOk: function(project) {
		if (project)
			this.dashboardItem.contextId = project.itemId == com.ibm.team.process.web.ui.internal.processAreaSelectionDialog.ALL_PROJECTS_ID ? Constants.CONTEXT_PUBLIC : project.itemId;
		else
			this.dashboardItem.contextId = this.dashboardItem.scopeItem.itemId;
		this.dashboardContainer.propertyChanged("shared", project);
		if (this.editMode == Constants.EDIT_AUTO_SAVE) {
			this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			if (project) {
				this._autoSave.autoSaveInput.checked = false;
				this._setAutoSave({target: this._autoSave.autoSaveInput});
			}
		} else if (this.editMode == Constants.EDIT_DEFERRED_SAVE) {
			if (project) {
				this.dashboardContainer.setEditMode(Constants.EDIT_AUTO_SAVE, true);
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
				this.dashboardContainer.setEditMode(Constants.EDIT_DEFERRED_SAVE, true);
			} else {
				this._autoSave.autoSaveInput.checked = "checked";
				this._setAutoSave({ target: this._autoSave.autoSaveInput });
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			}
		}
	},
	_deleteDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.delConfirmPrivate, [this.dashboardItem.title]);
		if (!confirm(message))
			return;
		this.dashboardContainer.propertyChanged("ondelete");	
	},
	_restoreDashboard: function(event) {
		if (event) 
			dojo.stopEvent(event);
		var message = dojo.string.substitute(messages.restoreDashboard, [Util.getScopeItemName(this.dashboardBundle)]);
		if (!confirm(message))
			return;
		this.dashboardContainer.restoreDashboard();
	},
	/*
	 * Called when the user changes the dashboard title.
	 */
	_onTitleChange: function(/* string */ newTitle) {
		this.dashboardItem.title = newTitle;
		this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		if (this.editMode != Constants.EDIT_DEFERRED_SAVE) {
			this.dashboardContainer.propertyChanged("title", this.dashboardItem.title);	
		}
	},
	showMessage: function(args) {
		if (this._messageVisible) this._clearMessage();
		if (args) {
			if (args.type) {
				var compArr = ["error","warn","info"];
				for (var i = 0; i < compArr.length; i++) {
					if (compArr[i]==args.type) dojo.addClass(this._messageArea,"message-"+compArr[i]);
					else dojo.removeClass(this._messageArea,"message-"+compArr[i]);
				}
				if(args.type == "error") {
					dojo.addClass(this._messageArea,"message-error");
				} else if (args.type == "warn") {
					dojo.addClass(this._messageArea,"message-warn");					
				} else if (args.type == "info") {
					dojo.addClass(this._messageArea,"message-info");					
				}
			}
			if (args.message) {
				this._message.appendChild(document.createTextNode(args.message));
			}
			if (args.actions && args.actions instanceof Array) {
				if (!this._messageActionHandlers) this._messageActionHandlers = new Array();
				for (var i = 0; i < args.actions.length; i++) {
					var a = document.createElement("a");
					a.innerHTML = args.actions[i].label;
					if (args.actions[i].callback) {
						a.href="#";
						var callback = this._getCallback(args.actions[i].callback);
						this._messageActionHandlers.push(this.connect(a,"onclick",callback));
					} else if (args.actions[i].href) {
						a.href = args.actions[i].href;
					}
					this._action.appendChild(a);
				}
			}
			this._messageArea.style.display = "block";
			this._messageVisible = true;
			if (!this._windowResizeEvent) {
				this._windowResizeEvent = ResizeNotifier.register({context: this, callback: this._onResizeHandler});
			}
			setTimeout(dojo.hitch(this,this._onResizeHandler),200);
		}
	},
	_getCallback: function(callback) {
		return function(event) {
			event.preventDefault();
			callback();
		};
	},
	hideMessage: function() {
		this._clearMessage();
		this._messageArea.style.display = "";
		this._messageVisible = false;
		ResizeNotifier.unregister(this._windowResizeEvent);
		this._windowResizeEvent = null;
		this.dashboardContainer.errorState = false;
	},
	_clearMessage: function() {
		while (this._messageActionHandlers && this._messageActionHandlers.length > 0) this.disconnect(this._messageActionHandlers.pop());
		while(this._action.firstChild)
			this._action.removeChild(this._action.firstChild);
		while(this._message.firstChild)
			this._message.removeChild(this._message.firstChild);
	},
	_onResizeHandler: function () {
		if (!this._messageNeedsRoom && this._messageArea.clientWidth < 160) {
			dojo.addClass(this._messageArea, "message-needs-room");
			this._messageNeedsRoom = true;
		} else if (this._messageNeedsRoom) {
			if (this.domNode && this.domNode.clientWidth - this._dashboardTitle.clientWidth - this._headerLinks.clientWidth - 43 >= 160) {
				dojo.removeClass(this._messageArea, "message-needs-room");
				this._messageNeedsRoom = null;
			}
		}
	},
	_onChange: function(/* string */ property) {
		// notify listeners that the content has changed
		this.propertyChanged(property);
	},
	setDirty: function (isDirty) {
		if (isDirty && !this.dashboardContainer.autoSave) {
			if (!this.dirtyIcon) {
				var span = document.createElement("span");
				span.innerHTML = "*";
				span.className = "dirtyStar";
				span.title = messages.dashboardModified;
				this._dashboardTitle.appendChild(span);
				this.dirtyIcon = span;	
			}
			this._saveButton.domNode.disabled = "";
		} else {
			if (this.dirtyIcon && this.dirtyIcon.parentNode) {
				this.dirtyIcon.parentNode.removeChild(this.dirtyIcon);
			}
			this.dirtyIcon = null;
			
			if (this._saveButton)
				this._saveButton.domNode.disabled = "disabled";
		}
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.MetaNavSharingDialogContent", com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialogContent, {
	postCreate: function() {
		this.inherited(arguments);
		var div = document.createElement("div");
		div.className = "com-ibm-team-dashboard-web-ui-metaNavSharing"
		this._checkbox = document.createElement("input");
		this._checkbox.type = "checkbox";
		this._checkbox.id = "MetaNavScopeDialogContent-sharedCheck";
		div.appendChild(this._checkbox);
		var label = document.createElement("label");
		dojo.attr(label, "for", "MetaNavScopeDialogContent-sharedCheck");
		label.innerHTML = messages.shareCheckboxLabel;
		div.appendChild(label);
		this._message = document.createElement("div");
		this._message.innerHTML = messages.shareVisibility;
		if (this.shared) {
			this._checkbox.checked = true;
			this._checkbox.defaultChecked = true;
		} else {
			this._treeNode.style.display = "none";
			this._message.style.display = "none";
		}
		this.domNode.insertBefore(div, this._treeNode);
		this.domNode.insertBefore(this._message, this._treeNode);
		this.connect(this._checkbox, "onclick", dojo.hitch(this,this._checkboxChanged));
	},
	
	_checkboxChanged: function(event) {
		if (this._checkbox.checked) {
			this._treeNode.style.display = "block";
			this._message.style.display = "block";
		} else {
			this._treeNode.style.display = "none";
			this._message.style.display = "none";
		}
	},
	
	_doOnOk: function() {
		if (!this._checkbox.checked)
			this.onOk(false);
		else
			this.inherited(arguments);
	}
});
})();