﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

import flash.filters.DropShadowFilter; 
 
class Tooltip extends ChartObject implements IMouseMoveEventListener {
	
	public var allowMove:Boolean;
	
	private var _tooltipHeight:Number;
	
	private var _tooltipWidth:Number;

	private static var _instanceCount:Number = 0;	
	
	private var _id:String;
	
	private var _text:String;
	
	private var _value:String;
	
	private var _isShowing:Boolean;

	private var _fadeInDelayHandler;
	
	private static var _LABEL_FIELD_FORMAT:TextFormat = Tooltip._initLabelTextFieldFormat();
	
	private static var _VALUE_FIELD_FORMAT:TextFormat = Tooltip._initValueTextFieldFormat();
	
	private static var _LABEL_FIELD_NAME:String = "tooltip_label_mc";

	private static var _VALUE_FIELD_NAME:String = "tooltip_value_mc";
	
	private static var _TOOLTIP_MOUSE_OFFSET_X:Number = 20.5;

	private static var _TOOLTIP_MOUSE_OFFSET_Y:Number = -2;

	private static var _TEXT_FIELD_WIDTH:Number = 100;

	private static var _TEXT_FIELD_HEIGHT:Number = 15;

	private static var _MARGIN_TOP:Number = 3;

	private static var _MARGIN_BOTTOM:Number = 5;

	private static var _MARGIN_LEFT:Number = 5;

	private static var _MARGIN_RIGHT:Number = 5;

	private static var _TOOLTIP_ARC_LENGTH:Number = 4;

	private static var _ALPHA_FADE_INCREMENT:Number = 15;
	
	private static var _BOX_FILL_COLOUR:Number = 0xfbfbec;
	
	private static var _BOX_OUTLINE_COLOUR:Number = 0xcdcd99;
	
	private static var _DROP_SHADOW_HEIGHT:Number = 3;

	private static var _MIN_LIMIT_COORD:Number = 3;
	
	private static var _TOOLTIP_DELAY_INTERVAL = 500; // ms
	
	
	public function Tooltip(text:String, value:String) {
		super();
		
		Tooltip._instanceCount++;
		this._id = "tooltip_mc_" + Tooltip._instanceCount;	
		
		this._text = text;
		this._value = value;
		this._isShowing = false;
		this.allowMove = true;
		
		this._tooltipWidth = 0;
		this._tooltipHeight = Tooltip._LABEL_FIELD_FORMAT.getTextExtent(text).textFieldHeight;
	}

	public function getTooltipWidth():Number {
		return this._tooltipWidth;
	}
	
	public function getTooltipHeight():Number {
		return this._tooltipHeight;
	}
	
	private static function _initLabelTextFieldFormat():TextFormat {
		var textFieldFormat = new TextFormat();
		textFieldFormat.color = 0x005c87;
		textFieldFormat.font = ChartArea.FONT;
		textFieldFormat.bold = true;
		textFieldFormat.size = 11;
		
		return textFieldFormat;
	}		

	private static function _initValueTextFieldFormat():TextFormat {
		var textFieldFormat = new TextFormat();
		textFieldFormat.color = 0x000000;
		textFieldFormat.font = ChartArea.FONT;
		textFieldFormat.size = 11;
		
		return textFieldFormat;
	}		
	
	public function draw(xCoord:Number, yCoord:Number):Void {
		this.mc = _root.createEmptyMovieClip(this._id, _root.getNextHighestDepth());
		this.mc._alpha = 0;
		
		// Label text field
		this._drawLabelField(xCoord, yCoord);
		// Value text field
		this._drawValueField(xCoord, yCoord);
		// Background box
		this._drawBox(xCoord, yCoord);
		// Drop shadow
		this._drawDropShadow();
	}
	
	private function _drawLabelField(x1:Number, y1:Number):Void {
		this.mc.createTextField(Tooltip._LABEL_FIELD_NAME, 
								this.mc.getNextHighestDepth(), 
								x1 + Tooltip._MARGIN_LEFT, 
								y1 + Tooltip._MARGIN_TOP, 
								Tooltip._TEXT_FIELD_WIDTH, 
								Tooltip._TEXT_FIELD_HEIGHT);
		this.mc[Tooltip._LABEL_FIELD_NAME]._visible = false;
		this.mc[Tooltip._LABEL_FIELD_NAME].text = this._text;
		this.mc[Tooltip._LABEL_FIELD_NAME].autoSize = "left";
		this.mc[Tooltip._LABEL_FIELD_NAME].setTextFormat(Tooltip._LABEL_FIELD_FORMAT);		
	}
	
	private function _drawValueField(x1:Number, y1:Number):Void {
		this.mc.createTextField(Tooltip._VALUE_FIELD_NAME, 
								this.mc.getNextHighestDepth(), 
								x1 + Tooltip._MARGIN_LEFT + this.mc[Tooltip._LABEL_FIELD_NAME]._width, 
								y1 + Tooltip._MARGIN_TOP, 
								Tooltip._TEXT_FIELD_WIDTH, 
								Tooltip._TEXT_FIELD_HEIGHT);
		this.mc[Tooltip._VALUE_FIELD_NAME]._visible = false;
		this.mc[Tooltip._VALUE_FIELD_NAME].text = ": " + this._value;
		this.mc[Tooltip._VALUE_FIELD_NAME].autoSize = "left";
		this.mc[Tooltip._VALUE_FIELD_NAME].setTextFormat(Tooltip._VALUE_FIELD_FORMAT);	
	}
	
	private function _drawBox(x1:Number, y1:Number):Void {
		
		this._tooltipWidth = this.mc[Tooltip._LABEL_FIELD_NAME]._width + 
							this.mc[Tooltip._VALUE_FIELD_NAME]._width +
							Tooltip._MARGIN_RIGHT;
		
		var x2:Number = x1 + this._tooltipWidth + Tooltip._MARGIN_RIGHT;
		var y2:Number = y1 + this._tooltipHeight + Tooltip._MARGIN_BOTTOM;

		this.mc.beginFill(Tooltip._BOX_FILL_COLOUR);
		this.mc.lineStyle(1, Tooltip._BOX_OUTLINE_COLOUR);
		
		// Start at top left corner
		this.mc.moveTo(x1, y1 + Tooltip._TOOLTIP_ARC_LENGTH);
		// Top left corner
		// TODO: Why is magic number 2 needed?
		this.mc.curveTo(x1 + 2, y1, x1 + Tooltip._TOOLTIP_ARC_LENGTH, y1);
		// Top edge
		this.mc.lineTo(x2 - Tooltip._TOOLTIP_ARC_LENGTH, y1);
		// Top right corner
		this.mc.curveTo(x2, y1, x2, y1 + Tooltip._TOOLTIP_ARC_LENGTH);
		// Right edge
		this.mc.lineTo(x2, y2 - Tooltip._TOOLTIP_ARC_LENGTH);
		// Bottom right corner
		this.mc.curveTo(x2, y2, x2 - Tooltip._TOOLTIP_ARC_LENGTH, y2);
		// Bottom edge
		this.mc.lineTo(x1 + Tooltip._TOOLTIP_ARC_LENGTH, y2);
		// Bottom left corner
		// TODO: Why is magic number 2 needed?
		this.mc.curveTo(x1 + 2, y2, x1, y2 - Tooltip._TOOLTIP_ARC_LENGTH);
		// Left edge
		this.mc.lineTo(x1, y1 + Tooltip._TOOLTIP_ARC_LENGTH);

		this.mc.endFill();			
	}
	
	private function _drawDropShadow():Void {
		var dropShadow:DropShadowFilter = new DropShadowFilter();
		dropShadow.blurX = 4;
		dropShadow.blurY = 4;
		dropShadow.distance = Tooltip._DROP_SHADOW_HEIGHT;
		dropShadow.angle = 45;
		dropShadow.quality = 3;
		dropShadow.alpha = 0.2;
		// Apply shadow filter
		this.mc.filters = [dropShadow];		
	}
	
	public function hide():Void {

		if (this._isShowing == false) {
			return;
		}

		ChartEngine.getInstance().removeMouseMoveListener(this._id);
		
		this._isShowing = false;
		
		this.mc._alpha = 0;
		
		//this._doFadeOut();
		
		this.mc[Tooltip._LABEL_FIELD_NAME]._visible = false;
		this.mc[Tooltip._VALUE_FIELD_NAME]._visible = false;
		
		this._disconnectFadeInDelayHandler();
	}
	
	public function show():Void {
		
		var context = this;

		// TODO: Refactor this method into two methods - one for mouse moves, one for static placement
		if (this.allowMove) {
			this.setPosition(_root._xmouse + Tooltip._TOOLTIP_MOUSE_OFFSET_X, 
							 _root._ymouse + Tooltip._TOOLTIP_MOUSE_OFFSET_Y);
			ChartEngine.getInstance().addMouseMoveListener(this._id, this);
		}
		
		if (this._isShowing) {
			return;
		}
		
		this._isShowing = true;

		this.mc[Tooltip._LABEL_FIELD_NAME]._visible = true;
		this.mc[Tooltip._VALUE_FIELD_NAME]._visible = true;

		this._connectFadeInDelayHandler();
	}

	private function _connectFadeInDelayHandler():Void {
		this._disconnectFadeInDelayHandler();
		
		var context:Tooltip = this;

		// TEST
		//trace(this._id + ": SetInterval");
		
		this._fadeInDelayHandler = setInterval(
			function() {
				// TEST
				//trace(context._id + ": DoInterval");
				context._doFadeIn();
				context._disconnectFadeInDelayHandler();
			}
			, Tooltip._TOOLTIP_DELAY_INTERVAL);
	}
	
	private function _disconnectFadeInDelayHandler():Void {
		if (this._fadeInDelayHandler) {
			// TEST
			//trace(this._id + ": ClearInterval");
			clearInterval(this._fadeInDelayHandler);
			this._fadeInDelayHandler = null;
		}
	}
	
	private function _doFadeIn():Void {

		var context = this;

	    this.mc.onEnterFrame = function() {
	    	
	    	if (context._isShowing == false) {
	    		delete this.onEnterFrame;
	    		return;
	    	}
	    	
	        this._alpha = this._alpha + Tooltip._ALPHA_FADE_INCREMENT;
	        
	        if (this._alpha >= 100) {
	            delete this.onEnterFrame;
	        }
	    };
	} 	

	private function _doFadeOut():Void {

		var context = this;

	    this.mc.onEnterFrame = function() {
	    	
	    	if (context._isShowing) {
	    		delete this.onEnterFrame;
	    		return;
	    	}
	    	
	        this._alpha = this._alpha - Tooltip._ALPHA_FADE_INCREMENT;
	        
	        if (this._alpha <= 0) {
	            delete this.onEnterFrame;
	        }
	    };
	} 	
	
	public function getLabelText():String {
		return this._text;
	}

	public function getLabelValue():String {
		return this._value;
	}
	
	public function setPosition(xCoord:Number, yCoord:Number):Void {
		
		if (!this.mc) {
			this.draw(0, 0);
		}
		
		var xPoint:Number = xCoord;
		var yPoint:Number = yCoord;
		
		var xLimit:Number = Stage.width -
							this._tooltipWidth -
							Tooltip._MARGIN_LEFT -
							Tooltip._DROP_SHADOW_HEIGHT;
		var yLimit:Number = Stage.height - 
							this._tooltipHeight -
							Tooltip._MARGIN_BOTTOM -
							Tooltip._DROP_SHADOW_HEIGHT;		
							
		// Prevent tooltip from being positioned off canvas vertically
		if (yPoint > yLimit) {
			yPoint = yLimit;
		} else if (yPoint < Tooltip._MIN_LIMIT_COORD) {
			yPoint = Tooltip._MIN_LIMIT_COORD;
		}
		
		// Prevent tooltip from being positioned off canvas horizontally
		if (xPoint > xLimit) {
			xPoint = xLimit;
		} else if (xPoint < Tooltip._MIN_LIMIT_COORD) {
			xPoint = Tooltip._MIN_LIMIT_COORD;
		}
		
		// Prevent tooltip from being rendered under the mouse pointer
		if (this._isMouseInTooltipRegion(xPoint, yPoint)) {
			var result:Object = this._getNoMouseTooltipRegion(xPoint, yPoint);
			xPoint = result.xPoint;
			yPoint = result.yPoint;
		}

		this.mc._x = Math.round(xPoint);
		this.mc._y = Math.round(yPoint);
	
	}
	
	private function _isMouseInTooltipRegion(xPoint:Number, yPoint:Number):Boolean {

		var xPointEnd:Number = xPoint + 
							   this._tooltipWidth +
							   Tooltip._MARGIN_LEFT +
							   Tooltip._DROP_SHADOW_HEIGHT;
		var yPointEnd:Number = yPoint +
							   this._tooltipHeight +
							   Tooltip._MARGIN_BOTTOM +
							   Tooltip._DROP_SHADOW_HEIGHT;

		// TEST
		//trace("======================");
		//trace("X:  " + xPoint);
		//trace("X2: " + xPointEnd);
		//trace("XM: " + _root._xmouse);
		//trace("Y:  " + yPoint);
		//trace("Y2: " + yPointEnd);
		//trace("YM: " + _root._ymouse);
							   
		if ((_root._xmouse >= xPoint) &&
			(_root._xmouse <= xPointEnd) &&
			(_root._ymouse >= yPoint) &&
			(_root._ymouse <= yPointEnd)) {
			
			return true;
		}

		return false;		
	}
	
	private function _getNoMouseTooltipRegion(xPoint:Number, yPoint:Number):Object {
		
		var result:Object = new Object();
		
		// Move tooltip left
		var xDelta:Number = xPoint +
							this._tooltipWidth +
							Tooltip._MARGIN_LEFT +
							Tooltip._DROP_SHADOW_HEIGHT -
							_root._xmouse;
							
		result.xPoint = xPoint - xDelta;
		result.yPoint = yPoint;

		if (result.xPoint >= Tooltip._MIN_LIMIT_COORD) {
			return result;
		}

		// Tooltip is rendered off canvas horizontally
		// Move tooltip down
		var yDelta:Number = yPoint +
						    this._tooltipHeight +
							Tooltip._MARGIN_BOTTOM +
							Tooltip._DROP_SHADOW_HEIGHT -
							_root._ymouse;
		result.xPoint = Tooltip._MIN_LIMIT_COORD;
		result.yPoint = yPoint + yDelta;

		// Note: Not checking whether the tooltip is rendered off canvas vertically
		
		return result;		
	}
	
	public function mouseMoved(xCoord:Number, yCoord:Number) {
		// TEST
		//trace(this._id + ": " + xCoord + " " + yCoord);
		this.setPosition(xCoord + Tooltip._TOOLTIP_MOUSE_OFFSET_X, 
						 yCoord + Tooltip._TOOLTIP_MOUSE_OFFSET_Y);
	}
	
	public function destroy():Void {
		
		ChartEngine.getInstance().removeMouseMoveListener(this._id);

		delete this.allowMove;
		delete this._tooltipHeight;
		delete this._tooltipWidth;
		delete this._id;
		delete this._text;
		delete this._value;
		delete this._isShowing;
		
		this._disconnectFadeInDelayHandler();
		delete this._fadeInDelayHandler;
		
		super.destroy();
	}			
	
}