/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.ObfUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ObfUtil = com.ibm.team.dashboard.web.util.internal.ObfUtil;
var ScopePreferenceProvider = com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab",
	com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab, {

	parseOnLoad: true,

	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.title = this._messages.settingsPrefs;
	},
	
	getNumberOfFields: function() {
		var length = this._hasScope() ? (this._hasScopeSubteams() ? 2 : 1) : 0;
		if (this._hasPrefs())
			length += this._settingsArea.viewletDefinition.preferenceDefinitions.length
		return length;
	},
	
	isRequired: function() {
		return this._hasScope() || this._hasPrefs();
	},
	
	_hasScope: function() {
		return this._settingsArea.viewletDefinition.scopeSensitive;
	},
	
	_hasScopeSubteams: function() {
		return this._settingsArea.viewletDefinition.scopeSubteams == "yes" || this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check";
	},
	
	_hasPrefs: function() {
		return this._settingsArea.viewletDefinition.preferenceDefinitions &&
				this._settingsArea.viewletDefinition.preferenceDefinitions.length > 0;
	},
	
	settingChanged: function(settingId) {
		var prefsToUpdate = this._settingsArea.viewlet.settingChanging(settingId, this._settingsArea.workingCopy);
		if (prefsToUpdate && prefsToUpdate.length) {
			for (var i = 0; i < prefsToUpdate.length; i++) {
				for (var j = 0; j < this._computedCombos.length; j++) {
					if (this._computedCombos[j] && prefsToUpdate[i] == this._computedCombos[j].select.name) {
						var value = this._computedCombos[j].select.options[this._computedCombos[j].select.selectedIndex].value;
						this._computedCombos[j].field.innerHTML = "";
						this.createFormField(this._computedCombos[j].data, this._computedCombos[j].field, value);
						this._computedCombos.splice(j,1);
						break;
					}
				}
			}
		}
	},
	
	initWorkingCopy: function() {
		if (this._hasScope()) {
			var workingCopy = this._settingsArea.workingCopy;
			var viewlet = this._settingsArea.viewlet;
			workingCopy.scope = viewlet.getScope();
			workingCopy.scopeItemId = viewlet.getScopeItem() ? viewlet.getScopeItem().itemId : null;
			if (this._hasScopeSubteams()) {
				var value = this._settingsArea.viewletHelper.scopeSubteams;
				if (!value && value != false)
					value = this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check";
				workingCopy.scopeSubteams = value;
			}
		}
		if (this._hasPrefs()) {
			var preferences = new Object();
			var prefDefs = this._settingsArea.viewletDefinition.preferenceDefinitions;
			var viewlet = this._settingsArea.viewlet;
			for (var i = 0; i < prefDefs.length; i++) {
				preferences[prefDefs[i].id] = this._settingsArea.viewlet.getPreference(prefDefs[i].id);
			}
			this._settingsArea.workingCopy.preferences = preferences;
		}
	},
	
	getDefaultValue: function(data) {
		if (data.id == "scope") {
			var result = {};
			result.type = this._settingsArea.viewletHelper.scope;
			if (!result.type)
				result.type = this._getApplicibleScope(this._settingsArea.dashboardItem.scope);
			var scopeItem = this._settingsArea.viewlet.getScopeItem();
			if (scopeItem)
				result.itemId = scopeItem.itemId;
			return result;
		} else if (data.id == "scopeSubteams") {
			var subteams = this._settingsArea.viewletHelper.scopeSubteams;
			if (!subteams && subteams != false)
				subteams = this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check";
			return subteams;
		}
		return this._settingsArea.workingCopy.preferences[data.id];
	},
	
	setup: function() {
		this._customPreferences = new Object();
		this._settingsArea.tabContainer.addChild(this);
		this.startup();
		if (this._hasPrefs())
			var data = this._settingsArea.viewletDefinition.preferenceDefinitions;
		else
			var data = [];
		if (this._hasScope() && (!data[0] || data[0].id != "scope")) {
			switch (this._settingsArea.viewletDefinition.appliesTo) {
				case Constants.SCOPE_APPLIES_TO_TEAMS:
					var name = this._messages.settingsScopeTeam;
					break;
				case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
					var name = this._messages.settingsScopeUser;
					break;
				case Constants.SCOPE_APPLIES_TO_PROJECTS:
					var name = this._messages.settingsScopeProject;
					break;
				case Constants.SCOPE_APPLIES_TO_PROCESS:
					var name = this._messages.settingsScopeProcess;
					break;
				default:
					var name = this._messages.settingsScopeAll;
			}
			if (this._hasScopeSubteams())
				data.unshift({ name: this._messages.settingsScopeSub, preferenceType: "BOOLEAN", id: "scopeSubteams", description: this._messages.settingsScopeSubDesc});
			data.unshift({
				id: "scope",
				name: name,
				preferenceType: "CUSTOM",
				required: true
			});
		}
		this.domNode.appendChild(this.createFormElements(data));
	},
	
	saveSettings: function(newSettings) {
		// saved form is array of objects
		if (this._hasPrefs()) {
			if (this._customPreferences) {
				for (var i in this._customPreferences)
					this._getPreferenceProvider(i).save();
			}
			var prefs = new Array();
			var wcPrefs = this._settingsArea.workingCopy.preferences;
			for (var i in wcPrefs) {
				prefs.push({ id: i, value: wcPrefs[i] });
			}
			this._settingsArea.viewletHelper.preferences = prefs;
		}
		if (this._hasScope()) {
			this._settingsArea.viewletHelper.scope = this._settingsArea.workingCopy.scope;
			this._settingsArea.viewletHelper.scopeItem = this._settingsArea.workingCopy.scopeItemId ? { itemId: this._settingsArea.workingCopy.scopeItemId } : null;
			if (this._hasScopeSubteams())
				this._settingsArea.viewletHelper.scopeSubteams = this._settingsArea.workingCopy.scopeSubteams;
		}
	},
	
	_updateCustomPreferenceLabel: function(prefId, value) {
		if (this._customPreferences[prefId]) {
			if (this._customPreferences[prefId].timeout)
				clearTimeout(this._customPreferences[prefId].timeout);
			var labelHTML = this._getPreferenceProvider(prefId).getLabel(value);
			if (!labelHTML) {
				labelHTML = this._getPreferenceProvider(prefId).getLoadingLabel();
				this._customPreferences[prefId].timeout = setTimeout(dojo.hitch(this, function() {
					this._updateCustomPreferenceLabel(prefId, value);
					}), 250);
			}
			this._customPreferences[prefId].label.innerHTML = labelHTML;
		}
	},
	
	_showCustomPreferenceChooser: function(event){
		dojo.stopEvent(event);
		
		this._getPreferenceProvider(event.target.name).createChooser(
			dojo.hitch(this,this._handleCustomChange), 
			this._settingsArea.workingCopy,
			this._settingsArea.viewlet);
	},
	
	_getPreferenceProvider: function(prefId) {
		if (prefId == "scope") {
			if (!this._scopeProvider) {
				// ask the viewlet for its scope preference provider
				// (handles remote viewlet case properly)
				var constructor = this._settingsArea.viewletContainer.getScopePreferenceProviderClass();
				this._scopeProvider = new constructor({
					appliesTo: this._settingsArea.viewlet._definition.appliesTo
				});
			}
			return this._scopeProvider;
		}
		return this._settingsArea.viewlet.getPreferenceProvider(prefId);
	},
	
	createFormField: function(data, field, value){
		switch(data.preferenceType) {
		case Constants.SettingsArea.FIELD_INTEGER:
			// show empty string if not specified
			var stringValue = "";
			if (value) {
				stringValue += value;
			}
			var inputDiv = document.createElement("div");
			var textBox = new dijit.form.TextBox({name: data.id, value: stringValue, trim: true, id: this._settingsArea.id + "-" + data.id});
			textBox.startup();
			textBox.domNode.className += " integer";
			if (data.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handleTextboxChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handleTextboxChange);
			inputDiv.appendChild(textBox.domNode);
			field.appendChild(inputDiv);
			return textBox.domNode;
		case Constants.SettingsArea.FIELD_PASSWORD:
			var inputDiv = document.createElement("div");
			var textBox = new dijit.form.TextBox({type: "password",name: data.id, value: value, trim: true, id: this._settingsArea.id + "-" + data.id});
			textBox.startup();
			textBox.domNode.className += " string";
			if (data.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handlePasswordChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handlePasswordChange);
			inputDiv.appendChild(textBox.domNode);
			field.appendChild(inputDiv);
			return textBox.domNode;
		case Constants.SettingsArea.FIELD_CUSTOM:
			var inputDiv = document.createElement("div");
			dojo.addClass(inputDiv, "custom-container");

			// Label
			var label = document.createElement("span");
			label.className = "custom-label";
			if (!(data.id in this._customPreferences))
				this._customPreferences[data.id] = new Object();
			this._customPreferences[data.id].label = label;
			this._updateCustomPreferenceLabel(data.id, value);
			
			// Button
			var button = document.createElement("button");
			button.name = data.id;
			button.id = this._settingsArea.id + "-" + data.id;
			this.connect(button,"onclick", this._showCustomPreferenceChooser);
			button.innerHTML = this._messages.settingsPrefsEdit;
			if (data.startDisabled)
				button.disabled = true;
			var buttonSpan = document.createElement("span");
			buttonSpan.className = "custom-edit-button";
			buttonSpan.appendChild(button);

			inputDiv.appendChild(buttonSpan);
			inputDiv.appendChild(label);
			field.appendChild(inputDiv);
			return;
		}
		return this.inherited(arguments);
	},
	_handleCheckboxChange: function(event) {
		var box = event.target;
		if (box.name == "scopeSubteams")
			this._settingsArea.workingCopy.scopeSubteams = box.checked;
		else
			this._settingsArea.workingCopy.preferences[box.name] = box.checked;
		this._settingsArea.settingChanged(box.name);
	},
	_handleTextboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = box.value;
		this._settingsArea.settingChanged(box.name);
	},
	_handlePasswordChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = ObfUtil.menc(box.value);
		this._settingsArea.settingChanged(box.name);
	},
	_handleComboboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.preferences[box.name] = box.options[box.selectedIndex].value;
		this._settingsArea.settingChanged(box.name);
	},
	_handleCustomChange: function(value, prefId) {
		this._updateCustomPreferenceLabel(prefId, value);
		if (prefId == "scope") {
			if (value.itemId != this._settingsArea.workingCopy.scopeItemId) {
				this._settingsArea.workingCopy.scopeItemId = value.itemId;
				prefId = "scopeItem";
			}
			if (value.type != this._settingsArea.workingCopy.scope) {
				this._settingsArea.workingCopy.scope = value.type;
				prefId = "scope";
			}
		} else
			this._settingsArea.workingCopy.preferences[prefId] = value;
		this._settingsArea.settingChanged(prefId);
	},
	_getApplicibleScope: function (scope) {
		switch (this._settingsArea.viewletDefinition.appliesTo) {
			case Constants.SCOPE_APPLIES_TO_TEAMS:
				if (scope != DashboardConstants.SCOPE_TEAM_AREA) {
					scope = DashboardConstants.SCOPE_TEAM_AREA;
				}
				break;
			case Constants.SCOPE_APPLIES_TO_CONTRIBUTORS:
				if (scope != DashboardConstants.SCOPE_CONTRIBUTOR) {
					scope = DashboardConstants.SCOPE_CONTRIBUTOR;
				}
				break;
			case Constants.SCOPE_APPLIES_TO_PROJECTS:
				if (scope != DashboardConstants.SCOPE_PROJECT_AREA) {
					scope = DashboardConstants.SCOPE_PROJECT_AREA;
				}
				break;
			case Constants.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					scope = DashboardConstants.SCOPE_PROJECT_AREA;
				}
				break;
		}
		return scope;
	}
		
});

})();
