/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.utils.ResultUtil");

dojo.require("jazz.util.date");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

com.ibm.team.build.web.ui.internal.utils.ResultUtil = {
	getLabel: function(resultDto, includeDefId, includePersonal) {
		var label = resultDto.buildResult.label;
		if (!label) {
			label = resultDto.buildResult.buildState == "NOT_STARTED" ? messages.pending : messages.unlabeled;
		}
		if (includeDefId) {
			var defId = resultDto.buildDefinition && resultDto.buildDefinition.id || "";
			label = dojo.string.substitute(messages.buildLabel, [defId, label]);
		}
		if (includePersonal && resultDto.buildResult.personalBuild){ //Show requestor whether pending or not.
			var requestorName = "";
			if(resultDto.requestor && resultDto.requestor.name)
				requestorName = resultDto.requestor.name;
			label = dojo.string.substitute(messages.personalBuildLabel,[ label, requestorName]);
		}
		return label;
	},
	
	getDuration: function(result){
		var duration = result.buildResult.buildTimeTaken;
		if(!duration || duration == -1){ // In-progress state
			var start = result.buildResult.buildStartTime;
			if(start){
				duration = new Date() - new Date(start);
			}
		}
		return jazz.util.date.formatDuration(duration && duration > 0 ? duration : 0);
	},
		
	insertWordBreaks: function(input, symbols, skipLastPeriod){
		// put the input in an array of length 1
		var htmlArr = [input];
		var index;
		// if we need to skip the last period, we have to process them first (while we have a single string)
		if (skipLastPeriod) {
			symbols.sort(function(a,b) {
				return b == "." ? -1 : a == "." ? 1 : 0;
			});
		}
		// for each symbol, split the input array into n segments
		// use the segmented output as the input for the next symbol
		for (var i = 0; i < symbols.length; i++) {
			var result = [];
			for (var j = 0; j < htmlArr.length; j++) {
				while((index = htmlArr[j].indexOf(symbols[i])) != -1) {
					if (skipLastPeriod && symbols[i] == "." && index == htmlArr[j].lastIndexOf(symbols[i]))
						break;
					result.push(htmlArr[j].substring(0,index+1));
					htmlArr[j] = htmlArr[j].substring(index+1);
				}
				result.push(htmlArr[j]);
			}
			htmlArr = result;
		}
		// join all the segments together with a word break between each
		return htmlArr.join("<wbr/>");
	}
};
})();
