/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */

dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.RepositoryWorkspaceBrowse");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() {
	var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;;
	var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;
	var ScmUtil				= com.ibm.team.scm.web.client.util;
	var ScmClient			= com.ibm.team.scm.web.client.internal.ScmClient;
	var PARTIAL_VIEW_COUNT  = 10;
	
	/**
	 * Widget to display "My Repository Workspaces" links.
	 */
	dojo.declare(
		"com.ibm.team.scm.web.ui.internal.view.browse.RepositoryWorkspaceBrowse",
		[dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/RepositoryWorkspaceBrowse.html"),
			
			constructor: function() {
				this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
				this.controller= null;
				this._workspaceList= [];
			},
			
			populate: function(sideNav) {
				this._selectedWorkspaceId = "-1";	//No workspace currently selected.
				
				var self = this;
				var serviceObject = {
					success: function(workspaces) {
						if(workspaces && workspaces.items) {
							//Display the list of workspaces.
							var count = 0;
							self._hidePartialLinks = new Array();
							dojo.forEach(workspaces.items, function(x) {
								var workspaceRow = self.addWorkspace({text: x.workspace.name, id: x.workspace.itemId, href: ScmUtil.getBrowseWorkspaceURI(x.workspace.itemId)});
								count ++;
								
								if(count > PARTIAL_VIEW_COUNT) {
									self._hidePartialLinks.push(workspaceRow);
									dojo.addClass(workspaceRow, "navHidden");							
								}				
							});
						}
						else {
							//Display an empty list of workspaces.
							self.addNoWorkspaces();
						}
						sideNav.update();
					},
				
					failure: function(errorObject) {
						self._displayError(self.messages.RepositoryWorkspaceBrowser_0);
					}
				};

				var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
				
				//Get logged in contributor.
				var contrib = com.ibm.team.repository.web.client.session.getAuthenticatedContributor();
				var ownerItemId = contrib.itemId;
				
				var OWNER_PREFIX = "itemOid/com.ibm.team.repository.Contributor/";
				//A REST call to retrieve the workspaces specific to the logged in contributor.
				ScmClient.getWorkspaces(new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObject, "success", "failure"), {owner: OWNER_PREFIX + ownerItemId});
			},
			
			/*
			 * Highlight/Unhighlight workspaces according to workspace selection.
			 */
			refreshSelection: function(workspaceId) {
				//If there is a workspace previously selected, unhighlight that workspace
				if(this._selectedWorkspaceId!="-1") {
					var div = document.getElementById(this._selectedWorkspaceId);
					if (div) {
						div.className = "workspace-div-unselected";
					}
				}
				//Highlight the new selected workspace
				if(workspaceId!="-1") {
					var div = document.getElementById(workspaceId);
					if (div) {
						div.className = "workspace-div-selected";
					}
				}
				//Update the selection
				this._selectedWorkspaceId = workspaceId;
			},
			
			/*
			 * Display an error message if failed to retrieve workspaces.
			 */
			_displayError: function(message) {
				var div = document.createElement("div");
				div.className = "errorMessage";
				var text = document.createTextNode(message);
				div.appendChild(text);
				this.workspaceListDiv.appendChild(div);
			},
			
			/*
			 * Display that no workspaces exist for the given contributor.
			 */
			addNoWorkspaces: function() {
				var div = document.createElement("div");
				div.className = "emptyWorkspaceList";
				var text = document.createTextNode(this.messages.RepositoryWorkspaceBrowser_1);
				div.appendChild(text);
				this.workspaceListDiv.appendChild(div);
				this._hidePartialLinks = null;
			},

			/*
			 * Add a workspace <div> for the corresponding workspace.
			 */
			addWorkspace: function(workspace) {
				this._workspaceList.push(workspace);
	             
	            var workspaceDiv= document.createElement("div");
	            if(workspace.selected) {
					workspaceDiv.className= "workspace-div-selected";
				} else { 
					workspaceDiv.className= "workspace-div-unselected";
				
				}
	    		workspace.div= workspaceDiv;
	            this._addWorkspaceLink(workspaceDiv, workspace);
	            this.workspaceListDiv.appendChild(workspaceDiv);
				return workspaceDiv;
			},
	
			/*
			 * Add a workspace link to the list of repository workspaces.
			 */
			_addWorkspaceLink: function (parentDiv, workspace) {
				var div= document.createElement("div");
				
				div.className = "workspace-div";
				div.id = workspace.id;
				var workspaceText= workspace.text;
				var href= workspace.href;
				
				var img = document.createElement("img");
				img.src = ScmUtil.getIconSrc("workspace");
				img.className = "workspaceIcon";
	
				var anchor= document.createElement("a");
				anchor.href= (href ? href : "#");
				var text= document.createTextNode(workspaceText);
				
				anchor.appendChild(text);
				div.appendChild(img);
				div.appendChild(anchor);
				
				parentDiv.appendChild(div);
			},
			
		hasShowPartial: function() {
			return this._hidePartialLinks ? this._hidePartialLinks.length > 0 : false;
		},
		
		onShowPartial: function() {
			if (this._hidePartialLinks)
				for (var i = 0; i < this._hidePartialLinks.length; i++)
					dojo.addClass(this._hidePartialLinks[i],"navHidden");
		},
		
		onMaximize: function() {
			if (this._hidePartialLinks)
				for (var i = 0; i < this._hidePartialLinks.length; i++)
					dojo.removeClass(this._hidePartialLinks[i],"navHidden");
		}
	}
);

})();
