/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentController");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var CACHE_SIZE = 6;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentController", null, {
	
	_widgetCache: [],
	constructor: function(multipaneContentView) {
		this._multipaneContentView = multipaneContentView;
	},
	destroy: function() {
		dojo.forEach(this._widgetCache, function(child) {
				try {
					if(child.destroyWidget) {
						child.destroyWidget();
					} else {
						if(child.destroy) child.destroy();
					}
				} catch(e) {
					alert('Error destroying: ' + child);// ignore
				}
			});
	},
	_destroyWidget: function(child) {
		if (child) {
			try {
				if (child.destroyWidget) {
					child.destroyWidget();
				}
				else {
					if (child.destroy) 
						child.destroy();
				}
			} 
			catch (e) {
				alert('Error destroying: ' + child);// ignore
			}
		} 
	},
	/*
	 * Return the currently cached widgets
	 */
	getWidgetCache: function () {
		return this._widgetCache;
	},
	
	showNextView: function() {
		return this._multipaneContentView.showNextView();
	},
	
	
	// Find the cached version of the widget
	getCachedWidget: function(widgetId, widget, instanceId, isSimple, instanceItemId) {
		if(instanceId == null && instanceItemId==null) return widget;
		var result;
		var cacheSize = CACHE_SIZE;
		if (instanceItemId) {
			for (var i = 0; i < this._widgetCache.length; i++) {
				if (this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].widget && this._widgetCache[i].widget._workItem && this._widgetCache[i].widget._workItem.itemId == instanceItemId) {
					result = this._widgetCache[i].widget;
					break;
				}
			}
		}
		else {
			for (var i = 0; i < this._widgetCache.length; i++) {
				if (this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
					result = this._widgetCache[i].widget;
					break;
				}
			}	
		}
				
		if(isSimple) return result;
		
		if(!result) {
			if(widget == null) return null;
			cacheSize--; // subtract one because we will add it below.
			var cachedObject = { widgetId: widgetId, widget: widget, instanceId: instanceId, isDirty: false };
		}
		
		/* if the cache is too big, remove the last, but not this one  */
		var removeId=-1;
		while(removeId != null && this._getNonDirtyCacheSize() >= cacheSize) {
			var removeId = this._multipaneContentView.getOldestNonDirtyId( parseInt(instanceId), cacheSize);
			if(removeId != null ) {
				var found = false;
				for(var i=0; i< this._widgetCache.length; i++ ) {
					if(this._widgetCache[i].instanceId == removeId) {
						var w = this._widgetCache[i];
						this.removeCachedWidget(w.widgetId, w.instanceId);
						found = true;
						break;
					}
				}
				if(!found) break; // in case not in cache
			}
		}
		if(!result) {
				this._widgetCache.push(cachedObject);
				var isChanged = false;
				if (widget.workingCopy) {
					isChanged = widget.workingCopy.isChanged;
				} else {
						var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(instanceId);
						if(workingCopy)	isChanged = workingCopy.isChanged();
				}
				
				this._multipaneContentView._addItem(widgetId, widget, instanceId, isChanged);
				result = widget;
		} 
		return result;		
	},
	_getNonDirtyCacheSize: function() {
		var count = 0;
		for(var i=0;i<this._widgetCache.length; i++) {
			if(!this._widgetCache[i].isDirty && parseInt(this._widgetCache[i].instanceId) > 0) count++;
		}
		return count;
	},
	removeCachedWidget: function(widgetId, instanceId) {
		if(instanceId == null) return;
		var newCache = [];
		var removedWidget;
		
		for(var i=0;i<this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				removedWidget =  this._widgetCache[i].widget;
				break;
			}
		}
		
		
		this._multipaneContentView._removeItem(instanceId);
		

		if (removedWidget) {
			this._destroyWidget(removedWidget);
		}		
		
		for(var i=0;i<this._widgetCache.length; i++) {
			if(!(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId)) {
				newCache.push(this._widgetCache[i]);
			}
		}
		this._widgetCache = newCache;
	},
	setIsDirty: function(widgetId, widget, instanceId, isDirty) {
		for(var i=0;i<this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				this._widgetCache[i].isDirty = isDirty;
				return;
			}
		}

	}
	
});

})();