/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var Util = com.ibm.team.workitem.web.client.util;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="HtmlWidget" >' +
						'<span style="display:block" dojoAttachPoint="descriptionSPAN" dojoAttachEvent="onchange: _handleChange" ></span>' +
						'<span style="display:block" dojoAttachPoint="descriptionTextSPAN" dojoAttachEvent="onkeyup: _handleKeyUp" class="DescriptionTextSpan"></span>' +
						'</div>',					
		_isChanged: false,
		_wasChangeNotified: false,
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			var value = this._getAttributeValue();
			this._childWidgets = [];

			// Now value is the R/W text, which needs some manipulation
			if(value && value.content != null) {
				var content = dojo.trim(value.content);
			} else {
				var content = value == null ? "" : dojo.trim(value + ""); // Handle nonstring values;
			}

			// Links need to be resolved in the original read only value because the links refer to offsets 
			if(value && value.links && value.links.length > 0 ) {
				this._originalReadOnlyValue = EditorUtil.resolveLinks(content, value.links); // handle "bug 1234" in content
			} else  {
				this._originalReadOnlyValue = content;
			}
						
			content = content.replace(/<br\/>/gi, "\n")
				
			this._isReadOnly = this.workingCopy.isGuest() || (content.length != 0 && content != "\n" ) || this._hasReadOnlyProperty();
			this._isTextWidget = this.creationSpec.originalWidgetName == 'com.ibm.team.workitem.web.ui.internal.view.editor.parts.TextAttributePart';
	
			this.descriptionTEXT = new com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget({parentController: this.parentController});
			this.descriptionTextSPAN.appendChild(this.descriptionTEXT.domNode);
			this.connect(this.descriptionTEXT.textarea, "onblur",  dojo.hitch(this, this._handleBlur));
			
			this._descriptionSpanClassName = "DescriptionSpan";
			this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty";
			if(this._modifiers.isSpanningColumns) {
				this._descriptionSpanClassName = "DescriptionSpan_TwoColumn";
				this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty_TwoColumn";
				this.descriptionSPAN.className = "DescriptionSpan_TwoColumn";
				this.domNode.className = "HtmlWidget_TwoColumn"
				this.descriptionTextSPAN.className = "DescriptionTextSpan_TwoColumn";
				this.descriptionSPAN.className = "DescriptionSpan_TwoColumn";
			}
			this.setSectionMenu = this.creationSpec.setSectionMenu == true;
			this.isDescription = this.creationSpec.isDescription == true;
						 
			if( !this._isTextWidget && !this.workingCopy.isGuest() && !this._modifiers.isString && (this._modifiers.isSpanningColumns || (this.isDescription && !this.setSectionMenu))) {
				    this._editAction = this.sectionLayout.getAction(this.messages.buttonEdit,  /* icon */  null, /* disable icon */ null, /* icon title */ this.messages.titleEdit,  this, "_toggleHTMLEditor", /* enable */ true);
					dojo.addClass(this._editAction, "SectionActionEmbedded");
					this.descriptionTextSPAN.appendChild(this._editAction);
			}
			
			this.setTextValue(content);

			if(this.setSectionMenu) {
				dojo.addClass(this.descriptionTEXT.textarea, "SelfSizingTextAreaTall");
			} else {
				dojo.addClass(this.descriptionTEXT.textarea, "SelfSizingTextAreaShort");
			}
			if (!this.workingCopy.isGuest()) {
				if (this.setSectionMenu && !this._editAction) {
					this._editAction = this.sectionLayout.addAction(this.messages.buttonEdit, /* icon */ null, /* disable icon */ null, /* icon title */ this.messages.titleEdit, this, "_toggleHTMLEditor", /* enable */ true);
				}
				else {
					if (!this._editAction && !this._hasReadOnlyProperty()) {
						this._isReadOnly = false;
					}
				}
			}
			
			this._setHTMLEditorState(this._isReadOnly);
			// Watch  for changes
			this._stopChangeInterval();
			this.changeInterval = setInterval(dojo.hitch(this, this._intervalCheckForChanges), 600);
			
			if(this._modifiers && this._modifiers.webHeight) {
				this.setHeight( this._modifiers.webHeight);
			}
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
				
		},
		getAttachmentInfo: function(number) {
			if(this.parentController && this.parentController.getAttachmentInfo) {
				return this.parentController.getAttachmentInfo(number);
			}
		},
		
		_hasReadOnlyProperty: function() {
			if (this.presentation && this.presentation.properties) {
				props= Util.buildMap(this.presentation.properties);
				return props.readonly == "true";
			}
			return false;
		},
		
		_handleAttributeChange: function(parms) {
			parms.attributeValue = parms.attributeValue || "";
			var value = this.getValue() || "";
			if(this.getValue() == parms.attributeValue) return;
			this.descriptionTEXT.setValue(parms.attributeValue);
			this._setHTMLEditorState(this._isReadOnly, /* skipFocus */ true );
		},
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.attributeId] });			
		},		
		destroyWidget: function() {
			if(this.descriptionTEXT) this.descriptionTEXT.destroy();
			this._stopChangeInterval();
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
			this.destroy();
		},
		getValue: function() {
			return this.getTextValue();
		},
		getTextValue: function() {
			return this.descriptionTEXT.getValue();
		},
		_intervalCheckForChanges: function(e, e, notifyEvenIfUnchanged, skipHandleChange) {
			this._checkForChanges(e, notifyEvenIfUnchanged, skipHandleChange);
		},
		_checkForChanges: function(e, notifyEvenIfUnchanged, skipHandleChange) {
			if (this._isValueChanged()) {
				this._stopChangeInterval();
				var e = {
					srcElement: this.descriptionTEXT
				};
				if(!skipHandleChange) {
					this._handleChange(e, notifyEvenIfUnchanged);
					return true;
				}
			}
			return false;
		},
		// IE adds returns to the values so strip them for compare
		_isValueChanged: function() {
			var oldValue = this._originalModifiedTextValue.replace(/\r/g,"");
			var newValue = this.getTextValue().replace(/\r/g,"");
			return oldValue != newValue
		},
		_stopChangeInterval: function() {
			if(this.changeInterval) {
				window.clearInterval(this.changeInterval);	
			}
		},
		setTextValue: function(value) {
			this._originalUnmodifiedTextValue = value;
			this._originalModifiedTextValue = this._cleanText(value);
			var text =this._originalModifiedTextValue;
			this.descriptionTEXT.setValue(this._originalModifiedTextValue);
		},
		_cleanText:function(value) {
			return EditorUtil.restoreTags(value);
		},
		_handleBlur: function(e){
			if(!this._isValueChanged()) return;
			this._wasChangeNotified = false; // force notify
			if(this._checkForChanges(e, /* notifyEvenIfUnchanged */ true, /* skipHandleChange*/ false)) return;
			this._handleChange(e, /*notifyEvenIfUnchange*/ true);
		},
		_handleKeyUp: function(e){
			if(!this._checkForChanges(e, /* notifyEvenIfUnchanged */ true, /* skipHandleChange*/ false)) return;
			this._handleChange(e, /*notifyEvenIfUnchange*/ false);
		},
		_handleChange: function(e, notifyEvenIfUnchanged) {
			this._isChanged= true;
			notifyEvenIfUnchanged = notifyEvenIfUnchanged && !this._wasChangeNotified;
			var isQuiet = this.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
			this._setWorkingCopyValue(this.getTextValue(), /* isInit */ false, isQuiet, notifyEvenIfUnchanged == true);
			this._wasChangeNotified = true;
			if(this.parentWidget){
				this.parentWidget._handleDirtyingEvent(e);
			}
		},
		_setWorkingCopyValue: function(value, isInit, isQuiet, notifyEvenIfUnchanged){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "content"],
				savedAttributePath: ["attributes", this.attributeId, "content"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				isQuiet: isQuiet,
				notifyEvenIfUnchanged: notifyEvenIfUnchanged,
				value: value
			});
		},
		_toggleHTMLEditor: function() {
			this._setHTMLEditorState(!this._isReadOnly);
			},
		_showHTMLEditor: function() {
			this._setHTMLEditorState(false);
			},
		_showHTMLPreview: function() {
			this._setHTMLEditorState(true);
			},
						
		_setHTMLEditorState: function(isReadOnly, skipFocus) {
			this._isReadOnly = isReadOnly;
			if(isReadOnly) {
				if (this._editAction) {
					dojox.data.dom.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonEdit));
				}

				// If the value is changed, use the user's value, else the original value with links added.
				// The _getTextValue will not have phrases resolved to links (e.g., "bug 1234")
				// since that is done on the server
				if(this._isValueChanged()) {
					var value = dojo.trim(this.getTextValue());
					if(dojo.isIE) {
						value = value.replace(/\n/g,"<br _synthetic_/>"); // preserve line feeds
						value = jazz.util.html.escape(value);
						value = value.replace(/\n/g,""); // remove line ends added by escape
						value = value.replace(/\&lt;br _synthetic_\/\&gt;/g,"\n"); // put original line feeds back
					} else {
						value = jazz.util.html.escape(value);
					}
				} else {
					var value = this._originalReadOnlyValue || "";	
				}

				var className = this._descriptionSpanClassName;
				if (value.length == 0) {
					className = this._descriptionSpanEmptyClassName;
					if(this._modifiers.noValueLabel) {
						value = this._modifiers.noValueLabel;
					} else {
						value = ""
					}
					var span = document.createElement("SPAN");
					if(value.length > 0) {						
						span.appendChild(document.createTextNode(value));
					} else {
						var div = document.createElement("DIV");
						div.innerHTML = "&nbsp;";
						span.appendChild(div);	
					}
				}
				else {
					var span = EditorUtil.getSafeTextSpan(value, this);
				}

				dojox.data.dom.replaceChildren(this.descriptionSPAN,span);
				this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(span, this.parentWidget._workItem)); // must be in dom
				this.descriptionSPAN.className = className
				this.descriptionSPAN.style.display = "block";
				this.descriptionTEXT.domNode.style.display = "none";
				
//				this.sectionLayout.setActionIcon(this._editDescriptionAction, ImagePool.EDIT, "Edit the description");
			} else {
				if (this._editAction) {
					dojox.data.dom.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonPreview));
				}
				
				if(this.domNode) {
					this.domNode.title = "";
				}
				this.descriptionSPAN.style.display = "none";
				this.descriptionTEXT.domNode.style.display = "";
				if(!skipFocus)	this.descriptionTEXT.focus();
			}
			
		},
		_handleClick: function(e){
			if(!this._isReadOnly) return;
			// If you click on an anchor, just follow the link
			if(e.target.tagName == "A") return;
			this._setHTMLEditorState(false);
		},
		setWidth: function(width) {
			this.descriptionTEXT.setWidth(width);
		},
		setHeight: function(height) {
			this.descriptionTEXT.setHeight(height);
		}
		
	}
);

})();
