/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;


dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;

com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil = {
		messages: dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages"),
		getImageUrl: function(imageName) {
			return ImageClient.getImageUrl(imageName);
		}, 
		getFirstValue: function(values, isString, returnEmpty, attrName) {
			var value;
			if(values != null && values.length > 0 ) {
				if (attrName) {
					value = values[0][attrName];
				}
				else {
					if (values[0].id) {
						value = values[0].id;
					}
					else {
						if (values[0].itemId) {
							value = values[0].itemId;
						}
						else 
							if (values[0].content) {
								value = values[0].content
							} else {
								value = values[0];
							}
					}
				}
			}
			if(value!=null) return value;
			if(isString && returnEmpty) return "";
			if(isString){
				return this.messages.labelDefaultNullValue;
			}
			return null;
		},
		isDebugMode: function() {
			return location.href.indexOf("debug=true") != -1;
		},
		setSpacerImage: function(image){
			if(!image) return;
			image.src = this.getUnknownIconUrl();
			image.style.visibility="hidden";
		},
		getUnknownIconUrl: function() {
			var url = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool.PIXEL;
			if(url && url.uri) return url.uri;
			return url;  
		},
		_bindValuesToSelect: function(widget, values, select, image, moreLabel, moreTitle) {
			var resultValue = "";
			var	selectedValueId  = select.value;
			if(selectedValueId == "") {
				if(widget.attributeValue) {
					if(widget.attributeValue.itemId) {
						selectedValueId=widget.attributeValue.itemId;
					} else {
						selectedValueId=widget.attributeValue.id ? widget.attributeValue.id : "";
					}
				}
			}
			dojox.data.dom.removeChildren(select);
		
			if (!moreLabel && (!values || values.length === 0)) {
				select.disabled= true;
				if(image) {
					image.src = this.getUnknownIconUrl();
					image.style.visibility="hidden";
				}
				EditorUtil._addOption(select, new Option(this.messages.labelDefaultNullValue, EditorUtil.UNCONFIGURED));
				return resultValue;
			} else {
				select.disabled= false;
			}
		
			for (var i = 0; i < values.length; i++) {
			
				var option = EditorUtil._createOption(values[i].label || values[i].name, values[i].id);
				option.valueObject = values[i];
				if (values[i].title) {
					option.title = values[i].title;
				}
				
				option[EditorUtil._getLabelProp()] = values[i].label || values[i].name;
				
				var isDefault = false;
				if (selectedValueId != "") {
					isDefault = selectedValueId && selectedValueId === values[i].id;
				}
				else {
					isDefault = i == 0;
				}
				
				if (isDefault) {
					resultValue = values[i];
					select.currentDefaultIndex = i;
					if (image) {
						image.src = values[i].iconUrl || EditorUtil.getUnknownIconUrl2();
						image.style.visibility = "visible";
					}
				}
				EditorUtil._addOption(select, option, isDefault);
				if (isDefault && values[i].title) {
					select.title = values[i].title;
				}
			}
			
		if(values.length == 0 && moreLabel) {
			var option = new Option(this.messages.labelDefaultNullValue, EditorUtil.UNCONFIGURED);
			option[EditorUtil._getLabelProp()] = EditorUtil.UNCONFIGURED;
			EditorUtil._addOption(select, option);
			select.currentDefaultIndex = 0;
		}

		if(moreLabel) {
			var option= EditorUtil._createOption(moreLabel, -1);
			option.title = moreTitle;
			EditorUtil._addOption(select, option, false);
		}
		return resultValue;
	},
		/*
		 * Build the sectionMenu menu
		 *  args = {
		 *   		domNode: ...,
		 *          parentWidget: ...,
		 *          
		 *          // Action class and menu class are passed from the widgets to avoid circular dependencies
		 *          actionClass:com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget,
		 *          menuClass: com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget,
		 *          actionSelector: ...,
		 *          actions: [ { 
		 *          			label: ...,
		 *          			iconSrc: ..., <optional>
		 *          			} ]
		 *     }
		 */
		createSectionMenu: function( args ) {
			
			if(args.sectionMenuWidget) {
				dojox.data.dom.removeChildren(args.domNode);
				args.sectionMenuWidget.destroyWidget();
			}
			
			var defaultAction;
			var menuActions = [];
			for(var i=0;i<args.actions.length;i++){
				var action = args.actions[i];
				var labelSelector = args.actionLabelSelector ? args.actionLabelSelector : "label" 
				var menuAction = {
					isEnabled: true,
					receiverObject: args.parentWidget,
					receiverSelector: args.actionSelector,
					label: action[labelSelector],
					iconSrc: action.iconUrl,
					value: action
				}
				if(args.defaultAction && args.actions[i].id == args.defaultAction.id) defaultAction = menuAction;
				menuActions.push(menuAction);
			}
			var actionNode = document.createElement("SPAN");
			var creationArgs = {
						parentWidget: args.parentWidget,
						actions: menuActions,
						defaultAction: defaultAction,
						label: args.menuLabel
					};
			var menuActionWidget = new args.actionClass(creationArgs, actionNode);
			
			creationArgs = {
						parentWidget: args.parentWidget,
						actionWidgets: [ menuActionWidget ]
					};
			
			return new args.menuClass(creationArgs, args.domNode);
		},
		_getCurrentSelectProps: function(select) {
			if(!select || select.selectedIndex < 0) return null;
			var currentOption= select.options[select.selectedIndex];
			var props= {
				label: currentOption[EditorUtil._getLabelProp()],
				id: select.value
			};
			if(currentOption.valueObject) {
				props.iconUrl = currentOption.valueObject.iconUrl;
			}
			return props;
		},
		/*
		 * Merge two objects. The second object has priority
		 */
		mergeObjects: function(object1, object2) {
			var result = object1;
			for(var key in object2) {
				result[key] = object2[key];
			}
			return result;
		},
		_getMailto: function(address, subject) {
			subject = subject ? subject : this.messages.labelDefaultEmailSubject;
			if(subject) {
//				return "mailto:" + address + "?subject=" + escape(subject);
				return "mailto:" + address + "?subject=" + subject;				
			}
			return "mailto:" + address;
		}
};

})();