/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.Menu"); //$NON-NLS-1$
dojo.require("dijit.ColorPalette"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.EventTopics"); //$NON-NLS-1$


dojo.require("dojo.string"); //$NON-NLS-1$ //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$ //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var EventTopics						= com.ibm.team.apt.web.ui.internal.EventTopics;
var TableRow						= null;
var CellEditor						= null;
var CellColorEditor					= null;

var replaceChildren					= dojox.data.dom.replaceChildren;

var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var COLORS= [
     { label: MSGS.COLORNAME_GREEN, color: "#8dc63f" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_ORANGE, color: "#f7941d" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_GOLD, color: "#e4d73d" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_TURQUOISE, color: "#5ab3a3" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_GRAY, color: "#797979" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_DARK_TURQUOISE, color: "#347aab" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_BLUE, color: "#4359a0" }, //$NON-NLS-1$
     { label: MSGS.COLORNAME_RED, color: "#c76363" } //$NON-NLS-1$
];

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration",  //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_messages: MSGS,
	
	__tableBody: null, 
	__btnAdd: null,
	__btnRemove: null,
	__btnUp: null,
	__btnDown: null,

	__viewEntries: null,
	__dirty: false,
	
	__selectedEntry: null,

	constructor: function(args){
		this.__viewEntries= [];
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	setColorizeConfig: function(configurations) {
		dojo.forEach(this.__viewEntries, function(entry) {
			entry.destroy();
		});
		
		this.__viewEntries= [];
		this.__selectedEntry= null;

		
		var configCopy= dojo.map(configurations, function(config) {
			return {
				label: config.label,
				enabled: config.enabled,
				color: config.color,
				priority: config.priority,
				expression: config.expression
			};
		});

		configCopy.sort(function(a,b) {
			return a.priority - b.priority;
		});

		dojo.forEach(configCopy, function(config) {
			this.__addEntry(config);
		}, this);

		this.__dirty= false;
	},
	
	getColorizeConfigs: function() {
		return dojo.map(this.__viewEntries, function(entry, index) { 
			var result= entry.getConfig();
			result.priority= index;
			return result;
		});
	},

	// ---- entry api ------------------------------------------------------------------------------------------------------
	
	_selectEntry: function(entry) {
		var selectedEntry= this.__selectedEntry;
		if (selectedEntry) {
			dojo.removeClass(selectedEntry.domNode, "selected"); //$NON-NLS-1$
		}
		this.__selectedEntry= entry;
		if (entry)
			dojo.addClass(entry.domNode, "selected"); //$NON-NLS-1$
		this.__updateActions();
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/config/templates/ColorizeConfiguration.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		this.__updateActions();
	},

	// ---- event handlers -------------------------------------------------------------------------------------------------
	
	__onAdd: function(event) {
		dojo.stopEvent(event);
		var newConfig= {
			label: MSGS.COLORIZE_NEW_ENTRY_LABEL,
			enabled: true,
			color: COLORS[0].color,
			priority: this.__viewEntries.length,
			expression: "" //$NON-NLS-1$
		};
			
		var newEntry= this.__addEntry(newConfig);
		this._selectEntry(newEntry);
		this.__setChanged(true);
	},
	
	__onRemove: function(event) {
		dojo.stopEvent(event);
		if (this.__selectedEntry) {
			this.__deleteEntry(this.__selectedEntry);
			this.__setChanged(true);
		}
	},
	
	__onDown: function(event) {
		dojo.stopEvent(event);
		if (this.__selectedEntry) {
			this.__moveDown(this.__selectedEntry);
			this.__setChanged(true);
		}
	},
	
	__onUp: function(event) {
		dojo.stopEvent(event);
		if (this.__selectedEntry) {
			this.__moveUp(this.__selectedEntry);
			this.__setChanged(true);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__addEntry: function(config) {
		var result= new TableRow(TableRow.constructorArgs(config, this));
		this.__tableBody.appendChild(result.domNode);
		this.__viewEntries.push(result);
		result.__changeListenerHandle= this.connect(result, "changed", "__entryChanged"); //$NON-NLS-1$ //$NON-NLS-2$
		this.__updateActions();
		return result;
	},
	
	__deleteEntry: function(entry) {
		var index= dojo.indexOf(this.__viewEntries, entry);
		if (index >= 0) {
			this.__viewEntries.splice(index, 1);
		}
		if (entry === this.__selectedEntry) {
			var newSelection= null;
			if (index >= 0 && this.__viewEntries.length > 0) {
				newSelection= this.__viewEntries[Math.max(index, this.__viewEntries.length - 1)];
			}
			this._selectEntry(newSelection);
		}
		
		if (entry.__changeListenerHandle) {
			this.disconnect(entry.__changeListenerHandle);
		}
		
		entry.destroy();
		this.__updateActions();
	},

	__moveDown: function(entry) {
		var index= dojo.indexOf(this.__viewEntries, entry);
		if (index >= 0 && index < (this.__viewEntries.length - 1)) {
			// update model
			var next= this.__viewEntries[index+1];
			this.__viewEntries[index+1]= entry;
			this.__viewEntries[index]= next;
			
			// update view
			var domNode= entry.domNode;
			var parent= domNode.parentNode;
			var successor= (index < this.__viewEntries.length - 2) ? this.__viewEntries[index+2].domNode : null;

			var removed= parent.removeChild(domNode);
			parent.insertBefore(removed, successor);

			this.__updateActions();
		}
	},
	
	__moveUp: function(entry) {
		var index= dojo.indexOf(this.__viewEntries, entry);
		if (index > 0) {
			// update model
			var prev= this.__viewEntries[index-1];
			this.__viewEntries[index-1]= entry;
			this.__viewEntries[index]= prev;
			
			// update view
			var domNode= entry.domNode;
			var parent= domNode.parentNode;
			var predecessor= prev.domNode;

			var removed= parent.removeChild(domNode);
			parent.insertBefore(removed, predecessor);

			this.__updateActions();
		}
	},
	
	__entryChanged: function(config) {
		this.__setChanged(true);
	},
	
	__setChanged: function(changed, configs) {
		this.__dirty= changed;
		if (changed) {
			dojo.publish(EventTopics.PLAN_PLANMODE_COLORIZE, [ this ]);
		}
	},
	
	__updateActions: function() {
		var index= -1;
		if (this.__selectedEntry)
			index= dojo.indexOf(this.__viewEntries, this.__selectedEntry);

		this.__setEnabled(this.__btnRemove, index != -1);
		this.__setEnabled(this.__btnUp, index > 0);
		this.__setEnabled(this.__btnDown, index >= 0 && index < this.__viewEntries.length - 1);
	},
	
	__setEnabled: function(node, enabled) {
		node.disabled= !enabled;
	},
	
	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration.constructorArgs= function() {
	return {
	};
};

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._TableRow", //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], { 

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	__config: null,
	__container: null,
	
	__cellEnabled: null,
	__cellColor: null,
	__cellLabel: null,
	__cellExpression: null,
	
	__enabledControl: null,
	__colorControl: null,
	__labelControl: null, 
	__expressionControl: null, 

	constructor: function(args) {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	getConfig: function() {
		return this.__config;
	},

	changed: function(config) {
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: '<tr dojoAttachEvent="onclick: __onSelect">' +  //$NON-NLS-1$
						'<td class="column enabled" dojoAttachPoint="__cellEnabled"></td>' + //$NON-NLS-1$
						'<td class="column color" dojoAttachPoint="__cellColor"></td>' +  //$NON-NLS-1$
						'<td class="column label" dojoAttachPoint="__cellLabel"></td>' +  //$NON-NLS-1$
						'<td class="column expression" dojoAttachPoint="__cellExpression"></td>' + //$NON-NLS-1$
					'</tr>', //$NON-NLS-1$

	postCreate: function() {
		this.__enabledControl= document.createElement("input"); //$NON-NLS-1$
		this.__enabledControl.setAttribute("type", "checkbox"); //$NON-NLS-1$ //$NON-NLS-2$
		this.__enabledControl.setAttribute("value", String(this.__config.enabled)); //$NON-NLS-1$
		this.__enabledControl.checked= this.__enabledControl.defaultChecked= this.__config.enabled;
		this.__cellEnabled.appendChild(this.__enabledControl);
		this.connect(this.__enabledControl, "onfocus", "__onSelect"); //$NON-NLS-1$ //$NON-NLS-2$
		this.connect(this.__enabledControl, "onchange", "__onEnablementChange"); //$NON-NLS-1$ //$NON-NLS-2$

		this.__colorControl= new CellColorEditor(CellColorEditor.constructorArgs(), this.__cellColor);
		this.__colorControl.setValue(this.__config.color);
		this.connect(this.__colorControl, "onChange", "__onColorChange"); //$NON-NLS-1$ //$NON-NLS-2$

		this.__labelControl= new CellEditor(CellEditor.constructorArgs(), this.__cellLabel);
		this.__labelControl.setValue(this.__config.label);
		this.connect(this.__labelControl, "onChange", "__onLabelChange"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.__expressionControl= new CellEditor(CellEditor.constructorArgs(), this.__cellExpression);
		this.__expressionControl.setValue(this.__config.expression);
		this.connect(this.__expressionControl, "onChange", "__onExpressionChange"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	// ---- event handlers -------------------------------------------------------------------------------------------------
	
	__onSelect: function(event) {
		this.__container._selectEntry(this);
	},
	
	__onEnablementChange: function() {
		this.__config.enabled= this.__enabledControl.checked;
		this.changed(this.__config);
	},

	__onColorChange: function(color) {
		this.__config.color= color;
		this.changed(this.__config);
	},

	__onLabelChange: function(label) {
		this.__config.label= label;
		this.changed(this.__config);
	},

	__onExpressionChange: function(expression) {
		this.__config.expression= expression;
		this.changed(this.__config);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._TableRow.constructorArgs= function(config, listControl) {
	return {
		__config: config,
		__container: listControl
	};
};

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellColorEditor", //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], { 

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_messages: MSGS,
	_transparentImageUrl: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/images/transparent.gif"), //$NON-NLS-1$ //$NON-NLS-2$

	__color: null,
	
	__colorIcon: null,
	__actionDropDown: null, 
	
	__menuCloseOnBody: null,
	__menu: null,
	
	constructor: function(args) {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	setValue: function(value) {
		this.__color= value;
		dojo.style(this.__colorIcon, "backgroundColor", value); //$NON-NLS-1$
	},

	getValue: function() {
		return this.__color;
	},
	
	onChange: function(value) {
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: '<td dojoAttachPoint="__cell" class="coloreditor">' + //$NON-NLS-1$
						'<div dojoAttachPoint="__colorIcon" class="coloricon"></div>' +  //$NON-NLS-1$
						'<a class="actionDropDownMenu" dojoAttachPoint="__actionDropDown" title="${_messages.COLORIZE_MENU_LABEL}" dojoAttachEvent="onclick: __onActionDropDownOpen">' +  //$NON-NLS-1$
							'<img src="${_transparentImageUrl}" alt="${_messages.COLORIZE_MENU_LABEL}"/>' +  //$NON-NLS-1$
						'</a>' +  //$NON-NLS-1$
					'</td>', //$NON-NLS-1$

	postCreate: function() {
		this.__actionDropDown.setAttribute("open", "false"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	// ---- event handlers -------------------------------------------------------------------------------------------------
	
	__onActionDropDownOpen: function(evt) {
		dojo.stopEvent(evt);
		
		if (this.__menu) {
			this.__closeMenu();
			return;
		}
		
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode, "com-ibm-team-apt-colorize-config"); //$NON-NLS-1$
		
		dojo.forEach(COLORS, function(color) {
			var menuItem= new dijit.MenuItem({
				label: color.label,
				onClick: dojo.hitch(this, function() { this.__changeColor(color.color); })
			});
			dojo.style(menuItem.iconNode, "backgroundColor", color.color);  //$NON-NLS-1$
			menu.addChild(menuItem);		
		}, this);
		menu.addChild(new dijit.MenuSeparator());
		menu.addChild(new dijit.MenuItem({
			label: MSGS.COLORIZE_MENUITEM_OPEN_COLORDIALOG,
			iconClass: "more", //$NON-NLS-1$
			onClick: dojo.hitch(this, function() { this.__selectColor(); })
		}));

		var pos = dojo.coords(this.__actionDropDown, true);
		
		this.__actionDropDown.setAttribute("open", "true"); //$NON-NLS-1$ //$NON-NLS-2$
		dijit.popup.open({
			popup: menu,
			parent: this,
			x: pos.x + 5,
			y: pos.y + pos.h - 2,
			onExecute: dojo.hitch(this, this.__closeMenu),
			orient: 'BL' //$NON-NLS-1$
		});
		
		//register event handler that causes menu to close
		this.__menuCloseOnBody= this.connect(this, "_onBlur", this.__closeMenu); //$NON-NLS-1$
		this.__menu = menu;
		
		menu.focus();
	},

	__closeMenu: function(evt) {
		if (evt)
			dojo.stopEvent(evt);

		this.disconnect(this.__menuCloseOnBody);
		dijit.popup.close(this.__menu);
		this.__menu = null;
		this.__actionDropDown.focus();
		this.__actionDropDown.setAttribute("open", "false"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	__changeColor: function(color) {
		if (this.__color != color) {
			this.__color= color;
			dojo.style(this.__colorIcon, "backgroundColor", color); //$NON-NLS-1$
			this.onChange(color);
		}
	},
	
	__selectColor: function() {
		var contentWidget= new dijit.ColorPalette();

		var floatingPane= new jazz.ui.Dialog({ 
			primaryTitle: MSGS.COLORIZE_COLORDIALOG_TITLE,
			contentNode: contentWidget.domNode,
			width: undefined //$NON-NLS-1$
		}); 
		
		this.connect(contentWidget, "onChange", dojo.hitch(this, function(selectedColor) { //$NON-NLS-1$
			this.__changeColor(selectedColor);
			floatingPane.close();
		}));
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellColorEditor.constructorArgs= function(color) {
	return {
		__color: color
	};
};

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellEditor", //$NON-NLS-1$
		[dijit._Widget, dijit._Templated], { 

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	__cell: null,
	__label: null,
	__edit: null,
	
	__currentValue: null,
	
	constructor: function(args) {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	setValue: function(value) {
		replaceChildren(this.__label, document.createTextNode(value));
		this.__edit.value= value;
		this.__currentValue= value;
	},

	getValue: function() {
		return this.__edit.value;
	},

	onChange: function(value) {
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: '<td dojoAttachPoint="__cell" class="celltexteditor">' + //$NON-NLS-1$
						'<span dojoAttachPoint="__label" dojoAttachEvent="onclick: __onBeginEdit"></span>' +  //$NON-NLS-1$
						'<input type="text" dojoAttachPoint="__edit" dojoAttachEvent="onblur: __onEndEdit"></input>' + //$NON-NLS-1$
					'</td>', //$NON-NLS-1$

	postCreate: function() {
	},
	
	// ---- event handlers -------------------------------------------------------------------------------------------------
	
	__onBeginEdit: function() {
	},
	
	__onEndEdit: function() {
		replaceChildren(this.__label, document.createTextNode(this.__edit.value));
		if (this.__currentValue != this.__edit.value) {
			this.__currentValue= this.__edit.value;
			this.onChange(this.__currentValue);
		}
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellEditor.constructorArgs= function() {
	return {};
};

TableRow= com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._TableRow;
CellColorEditor= com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellColorEditor;
CellEditor= com.ibm.team.apt.web.ui.internal.view.plan.config.ColorizeConfiguration._CellEditor;


})();
