/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");

(function(){
	
var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;
var HoverPopup= jazz.ui.HoverPopup;
	
dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/ErrorSection.html"),
	
	postCreate: function() {
		this._originalDisplay= this.domNode.style.visibility;
		this._hide();
	},
	destroy: function() {
		if(this._details) this._details.destroy();
		this.inherited(arguments);
	},	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	showReport: function(report) {
		if (report.severity == QueryClient.PROCESS_REPORT_OK) {
			this._hide();
		} else {
			this._showMessage(report.message);
			this._updateDetails(report);
			this._show();
		}
	},
	
	showError: function(message, error) {
		var report= { 
			severity: QueryClient.PROCESS_REPORT_ERROR,
			message: message,
			infos: [ {
				summary: message,
				description: (error.message || "")
			}]
			};
		this.showReport(report);
	},
	
	clearError: function() {
		this._hide();
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internal
	
	_showMessage: function(message) {
		dojox.data.dom.replaceChildren(this._error, document.createTextNode(message));
	},
	
	_updateDetails: function(report) {
		var infos= report.infos;
		if (infos && infos.length > 0) {
			var details= document.createElement("span");
			for (var i = 0; i < infos.length; i++) {
				
				var summary= document.createElement("b");
				summary.appendChild(document.createTextNode(infos[i].summary))
				
				var info= document.createElement("p");
				info.appendChild(summary);
				info.appendChild(document.createElement("br"));
				info.appendChild(document.createTextNode(infos[i].description));
				
				details.appendChild(info);
			}
			if(this._details) this._details.destroy();
			this._details = new HoverPopup({around: [this._error], html: details.innerHTML});
		}
	},
	
	_hide: function() {
		this.domNode.style.visibility= "hidden";
	}, 
	
	_show: function() {
		this.domNode.style.visibility= this._originalDisplay;
	}
	
});
	
})();
