/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var replaceChildren			= dojox.data.dom.replaceChildren;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_input: null,
	_labelProvider: null,
	_width: null,
	_height: null,
	
	_progressBar: null,
	_barContainer: null,
	_noWorkEstimatedLabel: null,
	
	// ---- api ------------------------------------------------------------------------------------------------------------	

	getLabelProvider: function() {
		return this._labelProvider;
	},
	
	update: function() {
		this.__doUpdate();
	},
	
	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/widgets/templates/ProgressBar.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		dojo.style(this._progressBar, "width", this.width); //$NON-NLS-1$
		dojo.style(this._progressBar, "font-size", this.height); //$NON-NLS-1$
		
		this.__doUpdate();
		
		new jazz.ui.HoverPopup({
			around: [this._progressBar], //$NON-NLS-1$
			createContents: dojo.hitch(this, this.__hoverCreateContents),
			orientation: "L" //$NON-NLS-1$
		});
		
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__hoverCreateContents: function() {
		this.__hoverNode= document.createElement("div"); //$NON-NLS-1$
		this.__hoverNode.innerHTML= this._labelProvider.getTooltipText(this._input);
		return this.__hoverNode;
	},
	
	__doUpdate: function() {
		
		if(this.__hoverNode){
			this.__hoverNode.innerHTML= this._labelProvider.getTooltipText(this._input);
		}
		
		var message= this._labelProvider.getMessage(this._input);
		if (message != null) {
			dojo.style(this._barContainer, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			dojox.data.dom.replaceChildren(this._noWorkEstimatedLabel, document.createTextNode(message));
		} else {
			dojo.style(this._noWorkEstimatedLabel, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._applyStyle(".barContainer .notEstimated", "height", 1 - this._labelProvider.getFillLevel(this._input)); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .estimated", "height", this._labelProvider.getFillLevel(this._input)); //$NON-NLS-1$ //$NON-NLS-2$ 

			var doneRange= this._labelProvider.getDoneRange(this._input);
			var projectionRange= this._labelProvider.getProjectionRange(this._input);
			
			var direction= this._labelProvider.getDeltaDirection(this._input);
			
			var isDone= doneRange[1] == 1;
			var canProject= !isDone && projectionRange[0] != -1 && projectionRange[1] != -1;
			                 
			var completed= 0;
			var delta= 0;
			if (isDone) {
				completed= 1;
			} else if (canProject) {
				completed= doneRange[1];
				delta= projectionRange[1];
			} else {
				completed= doneRange[1];
			}

			this._applyStyle(".barContainer .completed", "width", completed); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .difference", "left", completed); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .difference", "width", delta); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .open", "width", 1 - completed - delta); //$NON-NLS-1$ //$NON-NLS-2$
			
			dojo.query(".barContainer .difference", this._progressBar).addClass(direction < 0 ? "behind" : (direction > 0 ? "ahead" : "none")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ 
		}
	},

	_applyStyle: function(query, style, value) {
		dojo.query(query, this._progressBar).style(style, Math.round(value * 100) + "%"); //$NON-NLS-1$
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar.constructorArgs= function(progressInfo, labelProvider, width, height) {
	return {
		_input: progressInfo,
		_labelProvider: labelProvider,
		width: width || "100%", //$NON-NLS-1$
		height: height || "1em" //$NON-NLS-1$
	};
};

})();
