/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.viewers.TableViewer"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

(function() {

dojo.declare("com.ibm.team.apt.web.ui.internal.viewers.TableViewer", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	__input: null,
	__labelProvider: null,
	__contentProvider: null,
	__columnProvider: null,
	__tableStyles: null,
	__comperator: null,
	
	__tableBody: null,

	// ---- api ------------------------------------------------------------------------------------------------------------	

	getLabelProvider: function() {
		return this.__labelProvider;
	},
	
	getContentProvider: function() {
		return this.__contentProvider;
	},
	
	getColumnProvider: function() {
		return this.__columnProvider;
	},
	
	setComparator: function(comparator) {
		this.__comperator= comparator;
		
		this.refresh();
	},
	
	setInput: function(input) {
		this.__input= input;
		
		this.refresh();
	},
	
	refresh: function() {
		if (this.__input) {
			dojox.data.dom.removeChildren(this.__tableBody);
		
			this.__createRows(this.__input);
		}
	},
	
	setMessage: function(message) {
		dojox.data.dom.removeChildren(this.__tableBody);
		
		var row= document.createElement("tr"); //$NON-NLS-1$
		
		var td= document.createElement("td"); //$NON-NLS-1$
		dojo.addClass(td, "aptViewerColumn0"); //$NON-NLS-1$
		td.appendChild(document.createTextNode(message));
		row.appendChild(td);
		
		var columnProvider= this.getColumnProvider();
		var columnCount= columnProvider.getColumnCount();
			
		for ( var i = 1; i < columnCount; i++) {
			td= document.createElement("td"); //$NON-NLS-1$
			dojo.addClass(td, "aptViewerColumn" + i); //$NON-NLS-1$
			row.appendChild(td); //$NON-NLS-1$
		}		
		
		this.__tableBody.appendChild(row);
	},
	
	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/viewers/templates/TableViewer.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		for ( var i = 0; i < this.__tableStyles.length; i++) {			
			dojo.addClass(this.__table, this.__tableStyles[i]);
		}
		
		this.__createHeader();
	},
	
	destroy: function() {
		dojox.data.dom.removeChildren(this.__tableBody);
		
		this.getLabelProvider().destroy();
		
		this.inherited(arguments);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__createHeader: function() {
		var columnProvider= this.getColumnProvider();
		var count= columnProvider.getColumnCount();
		for ( var index = 0; index < count; index++) {
			var rowLine= document.createElement("th"); //$NON-NLS-1$
			this.__tableHeaderRowLine.appendChild(rowLine);
			
			var headerColumn= document.createElement("th"); //$NON-NLS-1$
			dojo.addClass(headerColumn, "aptViewerColumn" + index); //$NON-NLS-1$
			var label= columnProvider.getLabel(index);
			headerColumn.appendChild(document.createTextNode(label));
			this.__tableHeaderRow.appendChild(headerColumn);
		}
	},
	
	__createRows: function(element) {				
		var rows= this.getContentProvider().getChildren(element);
		rows= this.__sort(rows);
		for ( var i = 0; i < rows.length; i++) {
			var rowElement= rows[i];
			
			var row= this.__createRow(rowElement);

			this.__tableBody.appendChild(row)
		}
	},
	
	__sort: function(elements) {
		if (!this.__comperator) {
			return elements;
		} else {
			elements.sort(this.__comperator);
		}
		
		return elements;
	},
	
	__createRow: function(rowElement) {
		var result= document.createElement("tr"); //$NON-NLS-1$
		
		var columnProvider= this.getColumnProvider();
		var columnCount= columnProvider.getColumnCount();
			
		for ( var i = 0; i < columnCount; i++) {
			var column= this.__createColumn(rowElement, i);
			
			result.appendChild(column);
		}
		
		return result;
	},
	
	__createColumn: function(rowElement, index) {
		var result= document.createElement("td"); //$NON-NLS-1$
		dojo.addClass(result, "aptViewerColumn" + index); //$NON-NLS-1$

		var labelProvider= this.getLabelProvider();
		var content= labelProvider.getColumn(rowElement, index);
		result.appendChild(content);
		
		return result;
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.viewers.TableViewer.constructorArgs= function(tableStyles, labelProvider, contentProvider, columnProvider) {
	return {
		__tableStyles: tableStyles,
		__columnProvider: columnProvider,
		__labelProvider: labelProvider,
		__contentProvider: contentProvider
	};
};

})();
