/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008-2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.page.CqPageController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.page.RecordTypeManager");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.page.CqRecordController");
dojo.require("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "common");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqPageController");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqRecordStrings");

(function(){

var MAXCACHEDWIDGETS = 5; // cached widgets limit size, coincident with MAXRECENTITEMS in RecentView

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;

var messages, _init, _needInit;
var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.CqPageController",
	null,
{
	cachedWidgetIds: [], // array stored cached widget id order
	
	constructor: function(page){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqPageController");
		_init = false;
		_needInit = true;
		this._page = page;
		this._widgetCache = {};
		this._controllers = {};
		var subscribe = dojo.subscribe;
		this._subscriptions = [
			subscribe("CQ_WELCOME_MESSAGE", page, "updateWelcomeMessage"),
			subscribe("CQ_LOGIN_SUCCESS", function(message){
				page.updateWelcomeMessage({ authenticated: true });
			}),
			subscribe("CQ_RECORD_SELECTED", this, "_onRecordSelected"),
    		subscribe("CQ_BATCH_UPDATE_RECORD", this,"_onBatchUpdate"),	
			subscribe("CQ_RECORD_CLOSED", this, "_onRecordClosed"),
			subscribe("CQ_ITEMLABEL_CHANGED", this, "_handleItemLabelChanged")
		];
		this._connects = [
			dojo.connect(this, "setMainContent", function(props){
				page.getView("RECENT").addRecent(props);
			})
		];
		this.registerCommands();
	},
	
	registerCommands: function(){
		// Bind all of the actions identified in plugin.xml to corresponding
		// methods in this widget. Note that you can bind the actions to any
		// code you want anyway you want, we just use page widget methods here
		// for simplicity.
		//
		// Note that we must use the 'self' idiom because the value of 'this'
		// changes within the scope of the inner function.

		this._prevUrl = "#action="+cq.ACTION_PREFIX+"viewWelcome";

		var self = this;
		ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		
		/*
		 * default action is called when "ClearQuest Records" tab is clicked
		 */
		ActionRegistry.registerAction(cq.ACTION_PREFIX+"_showDefaultView", this, "_showDefaultView");
		
		dojo.forEach(
			[ "viewWelcome", "search", "showQueryResult", "showRecord", "newRecord" ],
			function(name){
				ActionRegistry.registerAction(cq.ACTION_PREFIX + name, function(kwArgs){
					// if this action is redirected from default action or invoked from recent view,
					// show cache and don't execute the action
					var executeAction = true;
					if(kwArgs.isRefresh == "false"){
						var cache = self._widgetCache[kwArgs.id];
						if(cache){
							self.setMainContent(cache, true);
							executeAction = false;
						}else if(name == "newRecord"){
							executeAction = false;
						}
					}
					
					// if cache was not shown, actually execute action
					if(executeAction){
						self[name](kwArgs);
					}
					
					// check views are initialized for all actions
					self.initializeViews();

					// store current URL so that it can be redirected when default action is called
					self._prevUrl = dojo.global.location.href;
				});
			}
		);
	},
	
	/*
	 * Do initialization of views
	 * - Setup create record dropdown
	 * - Populate favorites view
	 * - Populate navigator view
	 */
	initializeViews: function(){
		// check if it is already initialized or initialization has been processed or not
		// use _needInit flag since somehow initialize is requested twice
		// when going "CQ Record" tab > other tab > "CQ Record" tab
		if(!_needInit){ return; }
		
		_needInit = false;
		var self = this;
		loginController.process({
			callback: function(){
				if(!_init){
					self._initCreateRecordButton();
					self._page.getView("FAVORITES").initialize();
					self._page.getView("NAVIGATOR").initialize();
					_init = true;
				}
			},
			onCancel: function(){
				_needInit = true;
			}
		});
	},
	
	setCreateRecordButton: function(/* jazz.ui.Button */ submitButton){
		this._createButton = submitButton;
	},
	
	_initCreateRecordButton: function(){
		var submitButton = this._createButton;
		if(!submitButton){ return; }
		var self = this;
		com.ibm.rational.cm.web.ui.page.recTypeManager.getDefaultRecordType({
			onLoad: function(defaultRecType){
				if(!defaultRecType){ return; }
				var defaultRecId = defaultRecType.LocationId;
				if(defaultRecId){
					self._defaultRecordTypeID = defaultRecId;

					// add "Loading..." menu
					submitButton.populateDropDown([
						{
							label: messages.Loading,
							url: "javascript:void(0);",
							iconUrl: cq.getIconUrl("LOADING")
						}
					]);

					// connect to open menu event
					var connect = submitButton.connect(submitButton, "_openMenu", function(e){
						dojo["require"]("com.ibm.rational.cm.web.ui.page.SubmitMenuManager");
						var submitManager = com.ibm.rational.cm.web.ui.page.submitManager;

						submitManager.initialize(
							submitButton,
							connect,
							{
								label: defaultRecType.DisplayName,
								url: "#action="+cq.ACTION_PREFIX+"newRecord&type=" + encodeURIComponent(defaultRecId) + "&title=" + encodeURIComponent(defaultRecType.DisplayName),
								id: defaultRecId
							}
						);
						
						// actually fetch submit types and create menu
						submitManager.populateSubmitMenu();
					});
					
					// update create record button label
					var defaultRecTypeNode = dojo.byId("defaultRecType");
					if(defaultRecTypeNode){
						defaultRecTypeNode.innerHTML = defaultRecType.DisplayName;
					}
				}
			},
			onError: function(response, ioArgs){
				console.error("GetDefaultRecordType:" + response);
			}
		});
	},
	
	/*
	 * Show widget to main content area
	 * props is a JavaScript object which have the form:
	 * {
	 *	id : String (stableLocation)
	 *	type: String (LIST, RECORD, CHART, REPORT)
	 *	title : String
	 *	content: Widget or domNode
	 * }
	 */
	setMainContent: function(/* Object */ props, /* boolean */ isCache){
		this._page.setMainContent(props);
		
		if(isCache){
			var content = props.content;
			if(content && content.showCached){
				content.showCached();
			}
		}		
		// Add to cache.  Don't cache the object if the preventCache setting is set.
		// For example, preventCache=true for filter panels		
		if (!props.preventCache) {
			var id = props.id;
			if (id) {
				// remove the LRU cached widget, when the total number hits the limit
				this._ensureCashedWidgetSize(id);
				this._widgetCache[id] = props;
			}
		}
	},
	
	/*
	 * Gets cached widget properties for a given id
	 * {
	 *	id: String (stableLocation)
	 *	type: String ("RECORD"|"LIST"|"FULL_TEXT"|"CHART"|"REPORT")
	 *	title: String
	 *	content: Widget or domNode
	 * }
	 */
	getMainContent: function(/* String */ id){
		if(this._widgetCache[id]){
			return this._widgetCache[id];
		}
		return null;
	},

	// Registered action
	_showDefaultView: function(){
		// append &isRefresh=false so that action won't get executed
		// when switching to "ClearQuest Records" tab from other tab
		var url = this._prevUrl;
		if(!url.match(/&isRefresh=/)){
			url += "&isRefresh=false";
		}
		location.replace(url);
	},
	
	// Registered action
	viewWelcome: function(){
		this._page.showWelcomePage();
	},
	
	// Registered action
	showQueryResult: function(kwArgs){
		this._executeQuery({
			id: kwArgs.id,
			type: kwArgs.type,
			title: kwArgs.title
		});
	},
	
	_executeQuery: function(props){
		var self = this;
		cq.loadModule("com.ibm.rational.cm.web.ui.page.QueryResultsController", function(){
			if(!self._controllers.QUERY_RESULTS){
				self._controllers.QUERY_RESULTS = new com.ibm.rational.cm.web.ui.page.QueryResultsController(self);
			}
			dojo.publish("CQ_QUERY_EXECUTE", [props]);
		});
	},
	
	// record-related methods
	
	// Registered action
	newRecord: function(kwArgs){
		// create a new record widget and show in the main content pane
		var type = kwArgs.type || this._defaultRecordTypeID;
		if(type){
			if(kwArgs.ts){
				var props = this._widgetCache["rcm_new_record:" + type + "/" + kwArgs.ts]; // same as id generatio in CqRecordController 
				if(props){
					// already opened. 
					return;
				}
			}
			var container = this._createRecordContainer();
			var view = this._getRecordController().newRecord(type, container, kwArgs.ts);
			container.startup();
			this.setMainContent({
				id: view.id,
				type: "RECORD",
				content: container,
				title: kwArgs.title || messages.NewRecord
			});
			if(this._createButton.makeUrlsUnique){
				this._createButton.makeUrlsUnique();
			}						
		}
	},
	
	// Registered action
	showRecord: function(kwArgs){
		// check for cache and don't execute again if is already cached
		var id = kwArgs.id;
		var props = this._widgetCache[id];
		if(!props){
			var found = this._getRecordController().findRecordView(id)
			if (found){
				props = this._widgetCache[found.id];
			}
		}
		if(props){
			this.setMainContent(props, true);
		}else{
			// check if this is new record
			if (id.indexOf("rcm_new_record") == 0){
				return;
			}
			// create a new record widget and show in the main content pane
			var container = this._createRecordContainer();
			this._getRecordController().openRecordView(id, container);
			container.startup();
			props = {
				id: id,
				type: "RECORD",
				content: container,
				title: kwArgs.title
			}
			this.setMainContent(props, false);
		}
	},
	
	// Topic event
	_onRecordSelected: function(message){
		var rec = message.selectedArtifactIds[0].recordId;
		if(dojo.isArray(rec)) {
			rec = rec[0];
		}
		var title = message.title || messages.DefaultRecordTitle;
		// invoke action
		dojo.global.location.href = cq.getActionHref("RECORD", rec, title);
	},

	_onBatchUpdate: function(message){
		// create a new record widget and show in the main content pane
		var container = this._createRecordContainer();
		var recId = this._getRecordController()._onBatchUpdate(message, container);
		if(!recId)
			return;
		var _messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "cqRecordStrings");
		var title = _messages.titleForBatchUpdate;
		container.startup();
		props = {
			id: recId,
			type: "RECORD",
			content: container,
			title: title
		}
		this.setMainContent(props, false);
		// invoke action
		dojo.global.location.href = cq.getActionHref("RECORD", recId, title);
	},
	
	_onRecordClosed: function(message){
		var id = message.recordId;
		this._page.getView("RECENT").removeItem(id);	
		this._removeCachedWidget(id);
		this._showLastCachedWidget();
	},
	
	// internal methods for handling CqRecord widget
	_getRecordController: function(){
		if(!this._controllers.RECORD){
//			dojo["require"]("com.ibm.rational.cm.web.ui.page.CqRecordController");
			this._controllers.RECORD = new com.ibm.rational.cm.web.ui.page.CqRecordController();
			this._page.domNode.appendChild(this._controllers.RECORD.domNode);
		}
		return this._controllers.RECORD;
	},
	
	_createRecordContainer: function(){
		dojo["require"]("dijit.layout.BorderContainer");
		var container = new dijit.layout.BorderContainer({
			showCached: function(){
				if(this.hasChildren()){
					var c = this.getChildren()[0];
					// Assume the first child is CqRecord
					if(c.showCached) c.showCached();
				}
			}
		});
		container.domNode.style.left = "10px";
		container.domNode.style.height = "500px";
		return container;
	},
	
	_handleItemLabelChanged: function(message){
		var id = message.id;
		var title = message.title;

		this._page.getView("RECENT").updateLabel(id, title);

		// update cache
		var cachedInfo = this._widgetCache[id];
		if(cachedInfo){
			cachedInfo.title = title;
		}
	},
	
	//// search-related methods
	
	_handleQuickSearch: function(searchText, scope){
		checkNotNull(searchText, "searchText");
		var searchText = dojo.trim(searchText);
		var url = "#action="+cq.ACTION_PREFIX+"search&q=" + encodeURIComponent(searchText) + "&tq=" + Date.parse(new Date().toString());
		if(scope){
			url += "&scope=" + scope;
		}
		dojo.global.location.href = url;
	},
	
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Searches for work items matching the string specified by 'kwArgs.q'.
	 */
	search: function(kwArgs){
		checkNotNull(kwArgs.q, "kwArgs.q");
		var searchText = dojo.trim(kwArgs.q);
		var substitute = dojo.string.substitute;
		var scope = kwArgs.scope; // "fullText" or "byID"
		if(scope == "fullText"){
			if(loginController.getLoginInfo("isFullTextSearchEnabled") != true){
				alert(messages.FullTextNotEnabled);
				return;
			}
			this._executeQuery({
				id: searchText,
				type: "FULL_TEXT",
				title: substitute(messages.SearchResult, ['"'+searchText+'"']),
				recordTypes: []
			});
		}else{
			cq.xhrGet({
				url: "cqfind.cq",
				content: {
					action: "DoFindRecord",
					recordId: searchText,
					searchType: "BY_RECORD_ID"
				},
				onBegin: function(){
					PlatformUI.getWorkbench().showMessage(substitute(messages.SearchingFor, ['"'+searchText+'"']));
				},
				load: function(response, ioArgs){
					var recordId = response.id;
					if(dojo.isString(recordId)){
						if(recordId.length > 0){
							// record was found, so open that record
							dojo.publish("CQ_RECORD_SELECTED", [{
								selectedArtifactIds: [{
									recordId: recordId,
									hasMastership: true
								}],
								openNewWindow: 0
							}]);
						}else{
							// record was not found. response is { status: 'false', id:'' }
							alert(messages.RecordNotFound+": "+searchText);
						}
					}else{
						// other error occurred, so error message dialog will be shown
						cq.checkError(response);
					}
				},
				error: function(error, ioArgs){
					alert(dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "common").errMsg);
				},
				handle: function(){
					PlatformUI.getWorkbench().hideMessageArea();
				},
				preventCache: true
			});		
		}
	},
	
	destroyController: function(){
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
		dojo.forEach(this._connects, dojo.disconnect);
		for(var key in this._controllers){
			var controller = this._controllers[key];
			if(controller && controller.destroyController){
				controller.destroyController();
			}
		}
	},
	
	_showLastCachedWidget: function(){
		for(var w in this._widgetCache){
		}
		if(w){
			this.setMainContent(this._widgetCache[w], true);
		}else{
			this._showDefaultView();
		}
	},
	
	_removeCachedWidget: function(id){
		var prop = this._widgetCache[id];
		if(!prop) return;
		var widget = prop.widget;
		if (widget) {
			if (widget.destroyWidget) {
				widget.destroyWidget();
			}
			else {
				if (widget.destroy) 
					widget.destroy();
			}
		}
		delete this._widgetCache[id];
	},
	
	_ensureCashedWidgetSize: function(id){
		var idsLRU = this.cachedWidgetIds;
		if (this._widgetCache[id]) {
			var index = dojo.indexOf(idsLRU, id);
			idsLRU.splice(index, 1); // remove the existing id
			idsLRU.push(id); // re-add the id into the end of the array
		} else {
			idsLRU.push(id);
			if (idsLRU.length > MAXCACHEDWIDGETS) { // cached widget number hits the limit
				// remove the first item in ids
				this._removeCachedWidget(idsLRU[0]);
				idsLRU = idsLRU.shift();
			}
		}
	}
});

})();