/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rational.cm.web.ui.view.navigation.ListView");

dojo.require("dijit._Widget");
dojo.require("jazz.util.format");
dojo.require("com.ibm.rational.cm.web._base");

(function(){

var cq = com.ibm.rational.cm.web;
var FormatUtil = jazz.util.format;

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.navigation.ListView",
	dijit._Widget,
{
	constructor: function(args){
		this._items = [];
		this._navSection = null;
	},
	
	populate: function(navSection){
		this._navSection = navSection;
	},

	getItem: function(id){
		var ret = this._getItem(id);
		if(ret){
			return ret.item;
		}
		return null;
	},

	_getItem: function(id){
		for(var i = this._items.length - 1; i >= 0; i--){
			if(this._items[i].id == id){
				return { item: this._items[i], index: i };
			}
		}
		return null;
	},

	addItem: function(item){
		var navSection = this._navSection;
		if(!navSection){ return; }
		
		var id = item.id;
		
		// first check if item is already listed in this view
		var node = null;
		for(var i = this._items.length-1; i >= 0; i--){
			if(this._items[i].id == id) {
				node = this._items[i].node;
				this._items[i].touched = new Date().getTime();
				break;
			}
		}
		
		// node doesn't exist, so add a new node
		if(!node){
			var label = item.label;
			if(dojo.isArray(label) && label.length > 0) {
				// this is a workaround. label might be array in some case...
				// if non string value is passed to addSectionRow, it might cause serious problem.
				label = label[0];
			}
			var href = item.href;
			var iconUrl = item.iconUrl;
			
			var displayLabel = (label) ? FormatUtil.insertWordBreaks(label, 18) : "";
			var node = navSection.addSectionRow(displayLabel, href, iconUrl);
	
			this._items.push({
				id: id,
				label: label,
				href: href,
				iconUrl: iconUrl,
				node: node,
				touched: new Date().getTime()
			});
			this.onItemChanged(this._items);
		
			if(item.buttonInfo){
				dojo.forEach(item.buttonInfo, function(info){
					navSection.addArtifactCommand(node, info.tooltip, function(e){ e.preventDefault(); info.onClick(); }, info.iconClass, info.hoverCommand);
				});
			}
		}
		
		return node;
	},
	
	removeItem: function(id){
		var ret = this._getItem(id);
		if(!ret){ return; }
		var item = ret.item;
		var index = ret.index;
		if(index < 0 || !item){ return; }
		var node = item.node;
		if(!node){ return; }
		node.parentNode.removeChild(node);
		this._items.splice(index, 1);
		this.onItemChanged(this._items);
	},
	
	onItemChanged: function(/* Array */ items){},
	
	clearItems: function(){
		dojo.forEach(this._items, function(item, index){
			if(item){
				this.removeItem(item.id);
			}
		}, this);
	},
	
	_getQueryNode: function(id, queryStr){
		var item = this.getItem(id);
		if(item){
			var rootNode = item.node;
			if(rootNode){
				var nodes = dojo.query(queryStr, rootNode);
				if(nodes.length > 0){
					return nodes[0];
				}
			}
		}
		return null;
	},
	
	updateLabel: function(id, label){
		var labelNode = this._getQueryNode(id, ".link-text"); 
		if(labelNode){
			// labelNode.innerHTML = label;  // this causes IE script error
			labelNode.innerHTML = "";
			labelNode.appendChild(dojo.doc.createTextNode(label));
		}
		var item = this.getItem(id);
		if(item){
			item.label = label;
		}
	},
	
	setProcessing: function(id, processing){
		var iconUrl = null;
		if(processing){
			iconUrl = cq.getIconUrl("LOADING");
		}else{
			var item = this.getItem(id);
			if(item){
				iconUrl = item.iconUrl;
			}
		}
		if(iconUrl){
			this.updateIcon(id, iconUrl);
		}
	},
	
	updateIcon: function(id, iconUrl){
		var iconNode = this._getQueryNode(id, "IMG"); 
		if(iconNode){
			iconNode.src = iconUrl;
		}
	}
});

})();