/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel");

dojo.require("dijit.Tree");

(function(){
	
var cq = com.ibm.rational.cm.web;
var BASE_URL = "cqnav.cq";

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel",
	dijit.tree.ForestStoreModel,
{
	// Override dijit.tree.ForestStoreModel
	mayHaveChildren: function(/*dojo.data.Item*/ item){
		return (item && (
			(item == this.root) ||
			(this.store.isItem(item) && this.store.getValue(item, "type") == "FOLDER")
		));
	},
	
	// Override dijit.tree.ForestStoreModel
	// _getRootNodes instead of getChildren for root nodes
	getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary:
		// 		Calls onComplete() with array of child items of given parent item, all loaded.
		var root = this.root;
		if(parentItem === root){
			if(root.children){
				// already loaded, just return
				callback(root.children);
			}else{
				// root top level nodes
				this._getRootNodes(
					dojo.hitch(this, function(items){
						root.children = items;
						callback(items);
					}),
					onError
				);
			}
		}else{
			// fetch child nodes
			var store = this.store;
			var identity = store.getIdentity(parentItem);
			if(identity && identity.length > 0){
				var children = parentItem.children;
				if(children
					&& children.length > 0 && children[0].type != "HIDDEN"){
					callback(parentItem.children);
				}else{
					var self = this;
					cq.xhrGet({
						url: BASE_URL,
						content: {
							action: "getChildren",
							resourceId: identity
						},
						load: function(response, ioArgs){
							if(!response || response.STATUS != "true"){
								onError();
								return;
							}
							
							// add new items
							var items = [];
							dojo.forEach(response.data, function(item){
								items.push(self.newItem(item, parentItem));
							});
							store.save();
							
							callback(items);
						},
						error: function(response, ioArgs){
							onError();
						},
						onCancel: function(){
							self.setUnchecked(parentItem);
						},
						handleError: true,
						preventCache: true
					});
				}
			}
		}
	},
	
	// custom method to notify tree to set node state to "UNCHECKED"
	setUnchecked: function(/*dojo.data.Item*/ item){},
	
	// custom method
	_getRootNodes: function(callback, onError){
		if(this._rootLoading){ return; }
		var self = this;
		var store = this.store;
		cq.xhrGet({
			url: BASE_URL+"?action=getRootNodes",
			onBegin: function(){
				self._rootLoading = true;
			},
			load: function(response, ioArgs){
				if(!response || !response.items){
					onError();
					return;
				}
				
				// add new items
				store._loadFinished = true; // needed in order to avoid unnecessary action=getChildren call with null resourceId
				var items = [];
				dojo.forEach(response.items, function(item){
					item.children = [];
					items.push(self.newItem(item, self.root));
				});
				store.save();
				callback(items);
			},
			error: function(response, ioArgs){
				onError();
				self._rootLoading = false;
			},
			handleError: true,
			preventCache: true
		});
	},
	
	// Override dijit.tree.ForestStoreModel
	_requeryTop: function(){
/*
		// reruns the query for the children of the root node,
		// sending out an onSet notification if those children have changed
		var oldChildren = this.root.children || [];
		
		this._getRootNodes(dojo.hitch(this, function(newChildren){
			this.root.children = newChildren;

			// If the list of children or the order of children has changed...	
			if(oldChildren.length != newChildren.length ||
				dojo.some(oldChildren, function(item, idx){ return newChildren[idx] != item;})){
				this.onChildrenChange(this.root, newChildren);
			}
		}));
		*/ 
	}
});

/*
 * Gets tree model for the navigator tree
 * - make it a singleton so that it can be shared among multiple trees
 */
var _navModel;
var getNavigatorTreeModel = function(){
	if(!_navModel){
		// create data store
		dojo["require"]("com.ibm.rational.cm.web.ui.data.Store");
		var store = new com.ibm.rational.cm.web.ui.data.WriteStore({
			url: BASE_URL+"?action=getChildren",
			requestMethod: "POST"
		});
		var identifier = "stableLocation";
		store._features['dojo.data.api.Identity'] = identifier;
		store._itemsByIdentity = {}; // IMPORTANT! without this, error occurs when newItem
		var labelAttr = "title";
		store._labelAttr = labelAttr;
		store.format = function(dataObject){
			if(dataObject && dataObject.STATUS == "true"){
				return {
					identifier: identifier,
					label: labelAttr,
					items: dataObject.data
				}
			}else{
				return null;
			}
		}
	
		// create model
		_navModel = new com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel({
			store: store
		});
	}
	return _navModel;
}

/*
 * Returns properties required to instantiate navigator tree
 */
com.ibm.rational.cm.getNavigatorProperties = function(){
	return {
		model: getNavigatorTreeModel(),
		filter: {}, // needed to enable filter
		getIconClass: function(item, opened){
			if(!item || this.model.mayHaveChildren(item)){
				return opened ? "dijitFolderOpened" : "dijitFolderClosed";
			}else{
				var type = this.model.store.getValue(item, "type");
				return type ? "cqTree_"+type : "dijitLeaf";
			}
		},
		showRoot: false,
		openOnClick: true
	};
}

})();