/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimit");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit.form.NumberTextBox");
dojo.require("com.ibm.rational.cm.web.ui.view.common.Dialog");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimitDialogContent",
	[dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/resultset/templates/ResultSetLimitDialogContent.html"),
	
	origSoftLimit: null,
	softLimit: null,
	hardLimit: null,
	persistForSession: false,
	
	postMixInProperties: function(){
		this.i18nStr = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages", this.lang);
		this.inherited(arguments);
		this.origSoftLimit = this.softLimit;
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// setup validation for soft limit textbox
		var softLimitBox = dijit.byId(this.id+"_softLimit");
		this.connect(softLimitBox, "validate", dojo.hitch(this, function(isFocused){
			if(softLimitBox.state && softLimitBox.state.length > 0){ // disable [Rerun] button if state is "Error" or "Warning"
				this.rerunBtn.attr("disabled", true);
			}else if(softLimitBox.attr("value") != this.origSoftLimit){
				this.rerunBtn.attr("disabled", false);
			}
		}));
		// RATLC01061301 : Falcon Result Set Limit dialog: inconsistent use of commas for numbers
		softLimitBox.format = function(/*Number*/ value, /*Object*/ constraints){
			if(isNaN(value)){ return ""; }
			return value;
		};
	},
	
	/*
	 * Enable "Rerun" button when soft limit value is changed
	 */
	softLimitChanged: function(val){
		this.softLimit = val;
		this.rerunBtn.attr("disabled", false);
	},
	
	persistValueChanged: function(e){
		this.persistForSession = e.target.checked;
	},
	
	setSoftLimit: function(value){
		this.softLimit = value;
		this.origSoftLimit = value;
		dijit.byId(this.id+"_softLimit").attr("value", value);
		this.limitExplanation.innerHTML = dojo.string.substitute(this.i18nStr.msgResultsetLimit, [value]);
	},
	
	setHardLimit: function(value){
		this.hardLimit = value;
		dojo.byId(this.id+"_hardLimit").value = value;

		var softLimitBox = dijit.byId(this.id+"_softLimit");
		softLimitBox.constraints = {
			places: 0,
			min: 1
		};
		
		// RATLC01267295 : Query Result Set Limit: Inconsistent behavior between CQEC and Falcon vs. CQNative regarding hard limit=0 tests
		// hardLimit=0 means there's no hard limit so treat it specially
		if(value > 0){
			softLimitBox.invalidMessage = softLimitBox.rangeMessage = dojo.string.substitute(this.i18nStr.msgValidation, [1, value]);
			softLimitBox.constraints.max = value;
		}
	},
	
	showResultsClicked: function(e){},
	
	rerunClicked: function(e){}
});

// RATLC01235509 : Delay loading of scripts in CqQueryResultsService.js until they are actually needed
// moved from CqQueryResultsService
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimit",
	com.ibm.rational.cm.web.ui.view.common.Dialog,
{
	postCreate: function(){
		this.inherited(arguments);

		var contentWidget = this._content = new com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimitDialogContent({});
		var self = this;
		contentWidget.showResultsClicked = function(){
			self.hide();
		};
		contentWidget.rerunClicked = function(){
			self.rsGrid.sendRequest({
				startIndex: 1,
				usrSoftLimitValue: contentWidget.softLimit,
				persistForSession: contentWidget.persistForSession
			}, false);
			self.hide();
		};
		this.attr("content", contentWidget.domNode);
		this.titleNode.innerHTML = this.title = contentWidget.i18nStr.dlgTitle;

		this.resize({ w:500 });
	},
	
	setLimits: function(grid, softLimit, hardLimit){
		this.rsGrid = grid;
		this._content.setSoftLimit(softLimit);
		this._content.setHardLimit(hardLimit);
	},
	
	show: function(){
		this._content.rerunBtn.attr("disabled", true);
		this.inherited(arguments);
	}
});

com.ibm.rational.cm.web.ui.view.resultset.limitDialog = new com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimit();