/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.ItemSequenceManager"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.SequenceValue"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.OutOfOfficeItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Timespan"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.schedule.WorktimeScheduler"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.schedule.WorktimeIterator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.schedule.TimespanIterator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.schedule.ExclusionIterator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.schedule.UnionIterator"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;
var PlanItem						= com.ibm.team.apt.client.PlanItem;
var OutOfOfficeItem					= com.ibm.team.apt.client.OutOfOfficeItem;
var SequenceValue					= com.ibm.team.apt.client.SequenceValue;
var Duration						= com.ibm.team.apt.client.Duration;

var Timespan						= com.ibm.team.apt.client.internal.Timespan;
var Instant							= com.ibm.team.apt.client.internal.Instant;
var WorktimeScheduler				= com.ibm.team.apt.client.internal.schedule.WorktimeScheduler;

var WorktimeIterator				= com.ibm.team.apt.client.internal.schedule.WorktimeIterator;
var TimespanIterator				= com.ibm.team.apt.client.internal.schedule.TimespanIterator;
var ExclusionIterator				= com.ibm.team.apt.client.internal.schedule.ExclusionIterator;
var UnionIterator					= com.ibm.team.apt.client.internal.schedule.UnionIterator;

var SEQUENCE_VALUE_COMPARATOR= function(planItem1, planItem2) {
	return planItem1.getSequenceValue().compareTo(planItem2.getSequenceValue());
};

var RESOLVED_ITEMS_COMPARATOR= function(planItem1, planItem2) {
	var t1= planItem1.getResolutionDate();
	var t2= planItem2.getResolutionDate();

	if (t1 != null && t2 != null) {
		return t1.compareTo(t2);
	} else if (t1 == null && t2 == null) {
		return planItem1.getSequenceValue().compareTo(planItem2.getSequenceValue());
	} else {
		return (t1 == null ? 1 : 0) - (t2 == null ? 1 : 0);
	}
};

var DURATION_ZERO= new Duration(0);

dojo.declare("com.ibm.team.apt.client.internal.ItemSequenceManager", null, { //$NON-NLS-1$

	__plan: null,
	__resolvedItems: null,
	__scheduledItems: null,
	__inboxItems: null,
	__worktimeScheduler: null,
	__assignment: 0,
	__target: null,

	__schedulingCache: null,

	constructor: function(plan, contributor) {
		this.__plan= plan;
		this.__contributor= contributor;
		this.__target= plan.getIterationHierarchy().getElement();

		this.__resolvedItems= [];
		this.__scheduledItems= [];
		this.__inboxItems= {};
		
		this.__schedulingCache= {};
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	initialize: function(contributorInfo) {
		this.__runWriteLocked(function() {
			this._updateWorkTime(contributorInfo);
	
			this.__plan.accept(dojo.hitch(this, function(planElement) {
				if (planElement instanceof PlanItem && this.__isItemIncluded(planElement)) {
					if (planElement.isResolved()) {
						this.__resolvedItems.push(planElement);
					} else {
						if (!planElement.getSequenceValue().isNew()) {
							this.__scheduledItems.push(planElement);
						} else {
							this.__inboxItems[planElement.getElementKey()]= planElement;
						}
					}
				}
				return true;
			}));
	
			// sort the workitems with the stored sequence...
			this.__scheduledItems.sort(SEQUENCE_VALUE_COMPARATOR);
	
			this.__updateSchedulingCache(null);
			this.__updateResolvedCache();
		});
	},

	dispose: function() {
		this.__plan= null;
		this.__resolvedItems= null;
		this.__scheduledItems= null;
		this.__inboxItems= null;
		this.__schedulingCache= null;
		this.__worktimeScheduler= null;
	},
	
	getTotalWorkTime: function(targetIteration) {
		// targetIteration is not yet of interest, as this ItemSequenceManager implementation can
		// only handle a single iteration anyways 
		var startDate= this.__plan.getStartDate();
		var endDate= this.__plan.getEndDate();
		if(startDate == null || endDate == null)
			return 0;
		if (endDate.before(startDate))
			return 0;
		return Math.round(this.__worktimeScheduler.workTime(new Timespan(startDate, endDate)) * this.__assignment);
	},
	
	getRemainingWorkTime: function(targetIteration) {
		var startDate= this.__plan.getStartDate();
		var referenceTime= this.__plan.getReferenceTime();
		if (startDate == null || startDate.before(referenceTime))
			startDate= referenceTime;

		// targetIteration is not yet of interest, as this ItemSequenceManager implementation can
		// only handle a single iteration anyways 
		var endDate= this.__plan.getEndDate();
		if (endDate == null || endDate.before(startDate))
			return 0;

		return Math.round(this.__worktimeScheduler.workTime(new Timespan(startDate, endDate)) * this.__assignment);
	},
	
	getWorkStart: function() {
		if (this.__isDisposed())
			return null;
		
		return this.__worktimeScheduler.schedule(this.__getSchedulingReferenceTime(), DURATION_ZERO, true).start;
	},

	getScheduledItems: function() {
		return this.__scheduledItems;
	},
	
	getResolvedItems: function() {
		return this.__resolvedItems;
	},
	
	getInboxItems: function() {
		return this.__inboxItems;
	},

	findAt: function(when) {
		if (this.__scheduledItems.length > 0) {
			var firstItem= this.__scheduledItems[0];
			if (this.__schedulingCache[firstItem.getElementKey()].start.after(when)) {
				return firstItem;
			}

			var lastItem= this.__scheduledItems[this.__scheduledItems.length - 1];
			if (this.__schedulingCache[lastItem.getElementKey()].end.before(when)) {
				return lastItem;
			}
		}
		
		var low= 0;
		var high= this.__scheduledItems.length - 1;

		while (low <= high) {
			var mid= (low + high) >> 1;
			var value= this.__scheduledItems[mid];
			var scheduledTime= this.__schedulingCache[value.getElementKey()];

			if (scheduledTime.contains(when)) {
				return value;
			} else if (scheduledTime.start.after(when)) {
				high= mid - 1;
			} else {
				Assert.isTrue(scheduledTime.end.equals(when) || scheduledTime.end.before(when));
				low= mid + 1;
			}
		}
		return null;
	},

	unschedule: function(item, setAccepted) {
		if (this.__isDisposed() || !this.__contributor.equals(item.getOwner()))
			return;
		Assert.isLegal(this.__plan === item.getPlanModel());

		this.__runWriteLocked(function() {
			this.__doUnschedule(item, setAccepted, true);
		});
	},

	schedule: function(item) {
		if (this.__isDisposed() || !this.__contributor.equals(item.getOwner()))
			return;
		Assert.isLegal(this.__plan === item.getPlanModel());

		if (!this.__isItemIncluded(item))
			return;

		if (arguments.length == 1) {
			this.__runWriteLocked(function() {
				this.__doSchedule(item, true);
			});
		} else if (arguments.length == 2) {
			var when= arguments[1];
			this.__runWriteLocked(function() {
				Assert.isLegal(!when.before(this.__plan.getReferenceTime()));
				
				var candidate= this.findAt(when);
				
				var before= null;
				var after= null;
				if (candidate == null && this.__scheduledItems.length > 0) {
					before= this.__scheduledItems[this.__scheduledItems.length - 1];
				} else if (candidate != null) {
					var timespan= this.__schedulingCache[candidate.getElementKey()];
					if ((when.getMilliseconds() - timespan.start.getMilliseconds()) < (timespan.end.getMilliseconds() - when.getMilliseconds())) {
						after= candidate;
					} else {
						before= candidate;
					}
				}
				
				this.__doScheduleSpecific(item, before, after, true);
			});
		} else if (arguments.length == 3) {
			var before= arguments[1];
			var after= arguments[2];
			this.__runWriteLocked(function() {
				Assert.isLegal(item != before && item != after && before != after);
				this.__doScheduleSpecific(item, before, after, true);
			});

		}
	},

	// ---- PlanItem's API -------------------------------------------------------------------------------------------------

	_updateReferenceTime: function() {
		if (!this.__isDisposed()) {
			this.__runWriteLocked(function() {
				this.__updateSchedulingCache(null);
				this.__updateResolvedCache();
			});
		}
	},

	_updateWorkTime: function(workEnvironment) {
		if (!this.__isDisposed()) {
			this.__runWriteLocked(function() {
				var iterator= new WorktimeIterator(workEnvironment.workDays); 
				
				var absences= this.__plan.getAbsences(this.__contributor);
				if (absences.length > 0) {
					var outOfOfficeTime= dojo.map(absences, function(absence) {
						return new Timespan(absence.getStart(), absence.getEnd());
					});
					
					iterator= new UnionIterator([iterator, new ExclusionIterator(new TimespanIterator(outOfOfficeTime))]);
				}
				
				this.__worktimeScheduler= new WorktimeScheduler(iterator);
				this.__assignment= workEnvironment.assignment / 100.0;
				
				this.__updateResolvedCache();
				this.__updateSchedulingCache(null);
			});
		}
	},

	_getScheduledTime: function(item) {
		if (item instanceof PlanItem) {
			if (!this.__isItemIncluded(item))
				return null;

			return this.__isDisposed() ? null : this.__schedulingCache[item.getElementKey()];
		} else if (item instanceof OutOfOfficeItem) {
			return new Timespan(item.getStart(), item.getEnd());
		} else {
			throw new Error("Illegal argument"); //$NON-NLS-1$
		}
	},

	_itemAdded: function(item) {
		if (!this.__isItemIncluded(item))
			return;

		if (!this.__isDisposed()) {
			this.__runWriteLocked(function() {
				this.__removeOldScheduling(item);
				if (item.isResolved()) {
					this.__resolvedItems.push(item);
					this.__updateResolvedCache();
				} else {
					this.__doAddToScheduling(item);
					this.__updateSchedulingCache(item);
				}
			});
		}
	},
	
	_itemRemoved: function(item) {
		if (!this.__isDisposed()) {
			this.__runWriteLocked(function() {
				this.__removeOldScheduling(item);
			});
		}
	},

	_processWorkItemChangeEvent: function(planElementDelta) {
		if (!this.__isDisposed()) {
			this.__runWriteLocked(function() {
				var planItem= planElementDelta.getPlanElement();
				Assert.isLegal(planItem instanceof PlanItem);
				
				var targetDelta= planElementDelta.getAttributeDelta(PlanItem.TARGET);
				if (targetDelta != null) {
					var isOldTargetIncluded= this.__isTargetIncluded(targetDelta.getOldValue());
					var isNewTargetIncluded= this.__isTargetIncluded(targetDelta.getNewValue());
					
					if (isOldTargetIncluded && !isNewTargetIncluded) {
						this._itemRemoved(planItem);
						return;
					} else if (!isOldTargetIncluded && isNewTargetIncluded) {
						this._itemAdded(planItem);
						return;
					}
				}
				
				if (!this.__isItemIncluded(planItem))
					return;
				
				var schedulingChanged= false;
				var firstItemToRefresh= planItem;
				
				var resolvedDelta= planElementDelta.getAttributeDelta(PlanItem.RESOLVED);
				if (resolvedDelta != null) {
					if (resolvedDelta.getNewValue()) {
						this.__removeOldScheduling(planItem);
						this.__resolvedItems.push(planItem);
					} else {
						var itemIndex=dojo.indexOf(this.__resolvedItems, planItem);
						if (itemIndex != -1) {
							this.__resolvedItems.splice(itemIndex, 1);
						}
						this.__doUnschedule(planItem, false, false);
					}
					schedulingChanged= true;
				}
				
				if (planElementDelta.affectsAttribute(PlanItem.ORIGINAL_ESTIMATE) || 
						planElementDelta.affectsAttribute(PlanItem.CORRECTED_ESTIMATE) ||
						planElementDelta.affectsAttribute(PlanItem.TIMESPENT)) {
					schedulingChanged= true;
				}
				
				if (planElementDelta.affectsAttribute(PlanItem.SEQUENCE_VALUE)) {
					var oldIndex= this.__findScheduledIndex(planItem);
					if (oldIndex != -1) {
						this.__scheduledItems.splice(oldIndex, 1);
						schedulingChanged= true;
					}
					
					if (this.__inboxItems[planItem.getElementKey()]) {
						delete this.__inboxItems[planItem.getElementKey()];
						Assert.isTrue(oldIndex == -1);
						schedulingChanged= true;
					}
					
					var insertionIndex= this.__doAddToScheduling(planItem);
					schedulingChanged|= insertionIndex != -1;
					if (firstItemToRefresh != null && oldIndex != -1 && (insertionIndex == -1 || oldIndex < insertionIndex)) {
						// if the item got moved towards the back, we have to start refreshing where planItem was previously inserted
						// however, make sure that we do not produce an IIOBE when the last element was removed -> be defensive and do a full refresh instead
						firstItemToRefresh= oldIndex < this.__scheduledItems.length ? this.__scheduledItems[oldIndex] : null;
					}
					
				}
				if (schedulingChanged) {
					this.__updateSchedulingCache(firstItemToRefresh);
					this.__updateResolvedCache();
				}			
			});
		}			
	},

	_getLastSequenceValue: function() {
		if (this.__isDisposed() || this.__scheduledItems.length == 0) {
			return SequenceValue.INITIAL;
		}

		return this.__scheduledItems[this.__scheduledItems.length - 1].getSequenceValue(); 
	},


	// ---- Implementation -------------------------------------------------------------------------------------------------

	__doUnschedule: function(subject, setAccepted, saveSequenceValue) {
		var sequenceValue= setAccepted ? SequenceValue.FACTORY.successor(this._getLastSequenceValue()): SequenceValue.NEW;
		
		if (this.__isItemIncluded(subject)) {
			this.__removeOldScheduling(subject);
			this.__inboxItems[subject.getElementKey()]= subject;
		}
		subject.setSequenceValue(sequenceValue, saveSequenceValue);
		this.__updateSchedulingCache(null);
	},

	
	__doScheduleSpecific: function(subject, predecessor, successor, saveSequenceValue) {
		this.__removeOldScheduling(subject);

		Assert.isLegal(predecessor == null || predecessor.getPlanModel() === subject.getPlanModel());
		Assert.isLegal(successor == null || successor.getPlanModel() === subject.getPlanModel());

		if (successor != null && predecessor == null) {
			var afterIndex= this.__findScheduledIndex(successor);
			if (afterIndex > 0) {
				predecessor= this.__scheduledItems[afterIndex - 1];
			}
		} else if (predecessor != null && successor == null) {
			var beforeIndex= this.__findScheduledIndex(predecessor);
			if (beforeIndex != -1 && beforeIndex < this.__scheduledItems.length - 1) {
				successor= this.__scheduledItems[beforeIndex + 1];
			}
		}

		var predecessorValue= predecessor != null ? predecessor.getSequenceValue() : null;
		var successorValue= successor != null ? successor.getSequenceValue() : null;
		
		
		var toSchedule= [];
		if (predecessorValue != null && successorValue != null && predecessorValue.compareTo(successorValue) == 0) {
			for (var i= this.__findScheduledIndex(predecessor); i >= 0; i--) {
				var previousValue= predecessorValue;
				predecessor= this.__scheduledItems[i];
				predecessorValue= predecessor.getSequenceValue();
				if (previousValue.compareTo(predecessorValue) != 0) {
					break;
				}
				toSchedule.push(predecessor);
			}
			
			toSchedule.reverse();
			
			toSchedule.push(subject);

			for (var i= this.__findScheduledIndex(successor); i >= 0 && i < this.__scheduledItems.length; i++) {
				var previousValue= successorValue;
				successor=  this.__scheduledItems[i];
				successorValue= successor.getSequenceValue();
				if (previousValue.compareTo(successorValue) != 0) {
					break;
				}
				toSchedule.push(successor);
			}

			Assert.isTrue(predecessorValue == null || successorValue == null || predecessorValue.compareTo(successorValue) != 0);
		} else {
			toSchedule.push(subject);
		}

		for (var i= 0; i < toSchedule.length; i++) {
			this.__removeOldScheduling(toSchedule[i]);
		}

		
		this.__doSetSequenceValues(toSchedule, predecessorValue, successorValue, 0, toSchedule.length - 1, saveSequenceValue);

		var insertAfter= predecessorValue != null ? this.__findScheduledIndex(predecessor) + 1 : 0;
		this.__scheduledItems.splice.apply(this.__scheduledItems, [insertAfter, 0].concat(toSchedule));

		this.__updateSchedulingCache(toSchedule[0]);
	},

	__doSetSequenceValues: function(toSchedule, predecessorValue, successorValue, lowIndex, highIndex, saveSequenceValue) {
		var midIndex= (lowIndex + highIndex) >> 1;
		var midItem= toSchedule[midIndex];
		
		var midValue= SequenceValue.INITIAL;
		if (predecessorValue != null && !predecessorValue.isNew() && successorValue != null && !successorValue.isNew()) {
			midValue= SequenceValue.FACTORY.between(predecessorValue, successorValue);
		} else if (predecessorValue == null && successorValue != null && !successorValue.isNew()) {
			midValue= SequenceValue.FACTORY.predecessor(successorValue);
		} else if (predecessorValue != null && !predecessorValue.isNew() && successorValue == null) {
			midValue= SequenceValue.FACTORY.successor(predecessorValue);
		}

		midItem.setSequenceValue(midValue, saveSequenceValue);

		if (lowIndex < midIndex) {
			this.__doSetSequenceValues(toSchedule, predecessorValue, midValue, lowIndex, midIndex - 1, saveSequenceValue);
		}

		if (midIndex < highIndex) {
			this.__doSetSequenceValues(toSchedule, midValue, successorValue, midIndex + 1, highIndex, saveSequenceValue);
		}
	},

	__updateSchedulingCache: function(firstItem) {
		this.__plan.compoundChange(function(builder) {
			var oldValue, newValue, planItem;
			var i= firstItem != null ? this.__findScheduledIndex(firstItem) : 0;
			if (i == -1)
				i= 0;
			
			var predecessor= i > 0 ? this.__schedulingCache[this.__scheduledItems[i - 1].getElementKey()]: null;
			var nextStart= predecessor != null ? predecessor.end : this.__getSchedulingReferenceTime();
			for (; i < this.__scheduledItems.length; i++) {
				planItem= this.__scheduledItems[i];
				oldValue= this.__schedulingCache[planItem.getElementKey()];
				newValue= this.__worktimeScheduler.schedule(nextStart, planItem.getDuration(), true);
				
				if (oldValue == null || !oldValue.equals(newValue)) {
					this.__schedulingCache[planItem.getElementKey()]= newValue;
					builder.changed(planItem, PlanItem.SCHEDULED_TIME, oldValue, newValue);
				}
				nextStart= newValue.end;
			}
			
			for (var planItemKey in this.__inboxItems) {
				planItem= this.__inboxItems[planItemKey];
				oldValue= this.__schedulingCache[planItemKey];
				this.__schedulingCache[planItemKey]= null;
				if (oldValue != null) {
					builder.changed(planItem, PlanItem.SCHEDULED_TIME, oldValue, null);
				}
			}
		}, this);
	},

	__updateResolvedCache: function() {
		this.__plan.compoundChange(function(builder) {
			this.__resolvedItems.sort(RESOLVED_ITEMS_COMPARATOR);
			
			var thisItemEndTime= this.__plan.getReferenceTime();
			for (var i= this.__resolvedItems.length - 1; i >= 0; i--) {
				var planItem= this.__resolvedItems[i];
				var resolutionDate= planItem.getResolutionDate();
				if (resolutionDate != null) {
					thisItemEndTime= resolutionDate;
				}
				
				var newValue= this.__worktimeScheduler.schedule(thisItemEndTime, planItem.getDuration(), false);
				
				if (i > 0) {
					var previousItem= this.__resolvedItems[i - 1];
					var previousItemResolutionDate= previousItem.getResolutionDate();
					if (previousItemResolutionDate != null) {
						if (newValue.start.before(previousItemResolutionDate)) {
							newValue= new Timespan(previousItemResolutionDate, thisItemEndTime);
						}
					}
				}
				
				Assert.isNotNull(newValue);
				var oldValue= this.__schedulingCache[planItem.getElementKey()];
				this.__schedulingCache[planItem.getElementKey()]= newValue;
				
				if (oldValue == null || !oldValue.equals(newValue)) {
					builder.changed(planItem, PlanItem.SCHEDULED_TIME, oldValue, newValue);
				}
				thisItemEndTime= newValue.end;
			}
		}, this);
	},

	__removeOldScheduling: function(planItem) {
		this.__plan.compoundChange(function(builder) {
			var planItemKey= planItem.getElementKey();
			var scheduledIndex= this.__findScheduledIndex(planItem);
			if (scheduledIndex != -1) {
				Assert.isTrue(!this.__inboxItems[planItemKey]);
				this.__scheduledItems.splice(scheduledIndex, 1);
				var oldValue= this.__schedulingCache[planItemKey];
				delete this.__schedulingCache[planItemKey];
				if (oldValue != null)
					builder.changed(planItem, PlanItem.SCHEDULED_TIME, oldValue, null);
				
				if (scheduledIndex < this.__scheduledItems.length)
					this.__updateSchedulingCache(this.__scheduledItems[scheduledIndex]);
			} else {
				delete this.__inboxItems[planItemKey];
			}
		}, this);
	},

	__doAddToScheduling: function(planItem) {
		var result= -1;
		if (!planItem.getSequenceValue().isNew()) {
			result= this.__findSchedulingIndex(planItem);
			if (result < 0)
				// the insertion index should always be negative, as the sequence value should be unique
				// (but do not make an assertion out of it as this could happen when we reach the end of the double's accuracy)
				result= -result - 1;
			
			// be defensive: we make sure that adding the element will never throw an AIOOBE
			result= Math.min(this.__scheduledItems.length, result);
			this.__scheduledItems.splice(result, 0, planItem);
		} else {
			this.__inboxItems[planItem.getElementKey()]= planItem;
		}
		
		return result;
	},

	__findSchedulingIndex: function(planItem) {
		var result, low= 0, high= this.__scheduledItems.length - 1;

		while (low <= high) {
			result= (low + high) >> 1;
			var relation= SEQUENCE_VALUE_COMPARATOR(this.__scheduledItems[result], planItem);

			if (relation > 0) {
				high= result - 1;
			} else if (relation < 0) {
				low= result + 1;
			} else {
				return result;;
			}
		}
		return -(low + 1);
	},
	
	__getSchedulingReferenceTime: function() {
		var now= this.__plan.getReferenceTime();

		if (this.__plan.getStartDate() != null && now.before(this.__plan.getStartDate()))
			now= this.__plan.getStartDate();

		return now;
	},

	__findScheduledIndex: function(item) {
		return dojo.indexOf(this.__scheduledItems, item);
	},

	__isItemIncluded: function(item) {
		return this.__contributor.itemId == item.getOwner().itemId && this.__isTargetIncluded(item.getTarget());
	},

	__isTargetIncluded: function(target) {
		return target != null && target.itemId == this.__target.itemId;
	},

	__isDisposed: function() {
		return this.__plan == null;
	},
	
	__runWriteLocked: function(code) {
		// no real write lock, but a compound change on the model
		return this.__plan.compoundChange(code, this);
	},

	__sentinel: null // terminates this class definition
});

})();
