/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.plancheck.PlanItemCheckReport"); //$NON-NLS-1$

(function() {

var severityMap= {
	INFO: 0,
	WARNING: 1,
	ERROR: 2,
	FATAL_ERROR: 3
};

var compareSeverity= function(s1, s2) {
	return (severityMap[s1] || -1) - (severityMap[s2] || -1);
};

	
dojo.declare("com.ibm.team.apt.client.internal.plancheck.PlanItemCheckReport", null, { //$NON-NLS-1$
	
	__problems: null,
	
	constructor: function(planElement) {
		this.__planElement= planElement;
		this.__problems= [];
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	getPlanElement: function() {
		return this.__planElement;
	},
	
	addProblem: function(newProblem) {
		this.__problems= dojo.filter(this.__problems, function(problem) { return !problem.equals(newProblem); });
		this.__problems.push(newProblem);
	},

	hasProblems: function() {
		return this.__problems.length > 0;
	},

	getProblems: function() {
		return this.__problems;
	},

	getProblem: function(problemId) {
		for (var i= 0; i < this.__problems.length; i++) {
			if (this.__problems[i].getId() == problemId) {
				return this.__problems[i];
			}
		}
		
		return null;
	},
	
	clearProblems: function(problemIds) {
		this.__problems= dojo.filter(this.__problems, function(problem) { 
			return problemIds.indexOf(problem.getId()) == -1; 
		});
	},
	
	getMostSevereProblem: function() {
		var result= null;
		for (var i= 0; i < this.__problems.length; i++) {
			if (result == null || compareSeverity(result.getSeverity(), this.__problems[i].getSeverity()) > 0) {
				result= this.__problems[i];
			}
		}

		return result;
	},
	
	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;
		
		if (!this.__planElement.equals(other.__planElement))
			return false;

		if (this.__problems.length != other.__problems.length)
			return false;
		
		for (var i = 0; i < this.__problems.length; i++) {
			if (!this.__problems[i].equals(other.__problems[i]))
				return false;
		}
		
		return true;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

})();
