/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.model.EntryNavigator"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.EntryUtils"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.PrimaryLocationTag"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;
var EntryUtils						= com.ibm.team.apt.ui.model.EntryUtils;

var PrimaryLocationTag				= com.ibm.team.apt.ui.model.PrimaryLocationTag.INSTANCE;

dojo.declare("com.ibm.team.apt.ui.model.EntryNavigator", null, { //$NON-NLS-1$

	__readAccessor: null,
	__filterFn: false,
	
	constructor: function(readAccessor, filterFn) {
		this.__readAccessor= readAccessor;
		this.__filterFn= filterFn;
	},

	// ---- API: find entries ---------------------------------------------------------------------------------------------

	findPrimaryEntry: function(element) {
		return this.__firstEntryWithTag(this.__readAccessor.getElementEntries(element), PrimaryLocationTag);
	},

	findFirstEntryWithTag: function(element, tag) {
		return this.__firstEntryWithTag(this.__readAccessor.getElementEntries(element), tag);
	},

	findFirstEntryInSubtree: function(element, searchRoot) {
		var entries= this.__readAccessor.getElementEntries(element);
		
		for (var i= 0; i < entries.length; i++) {
			var entry= entries[i];
			if (this.__isIncluded(entry) && this.startsWith(entry, searchRoot)) {
				return entry;
			}
		}
		return null;
	},
	
	// ---- API: query elements --------------------------------------------------------------------------------------------

	parentElement: function(entry) {
		return EntryUtils.element(this.parentEntry(entry));
	},

	parentElement: function(entry, level) {
		return EntryUtils.element(this.parentEntry(entry, level));
	},

	childern: function(entry) {
		return this.__doGetElements(this.childEntries(entry));
	},

	siblings: function(entry) {
		return this.__doGetElements(this.siblingEntries(entry));
	},

	firstSibling: function(entry) {
		return EntryUtils.element(this.firstSiblingEntry(entry));
	},

	lastSibling: function(entry) {
		return EntryUtils.element(this.lastSiblingEntry(entry));
	},

	predecessor: function(entry) {
		return EntryUtils.element(this.predecessorEntry(entry));
	},

	successor: function(entry) {
		return EntryUtils.element(this.successorEntry(entry));
	},

	// ---- API: query entries ---------------------------------------------------------------------------------------------

	parentEntry: function(entry) {
		return this.__readAccessor.getParent(entry);
	},

	parentEntry: function(entry, level) {
		if (!level)
			return this.__readAccessor.getParent(entry);
		
		if (level < 0)
			throw new Error("Index out of bounds"); //$NON-NLS-1$

		for (var i= level; entry != null && i > 0; i--) {
			entry= this.__readAccessor.getParent(entry);
		}
		return entry;
	},

	parentEntryOfType: function(entry, typeClass) {
		while (entry != null) {
			if (entry.getElement() instanceof typeClass) {
				return entry;
			}
			entry= this.__readAccessor.getParent(entry);
		}
		return null;
	},

	childEntries: function(entry) {
		return this.__filterInvisible(this.__readAccessor.getChildren(entry));
	},

	siblingEntries: function(entry) {
		var parent= this.__readAccessor.getParent(entry);
		return parent != null ? this.__filterInvisible(this.__readAccessor.getChildren(parent)) : null;
	},

	firstSiblingEntry: function(entry) {
		var siblings= this.siblingEntries(entry);
		// if siblings != null then it must contain at least one entry -> itself
		return siblings ? siblings[0] : null;
	},

	lastSiblingEntry: function(entry) {
		var siblings= this.siblingEntries(entry);
		// if siblings != null then it must contain at least one entry -> itself
		return siblings ? siblings[siblings.length - 1] : null;
	},

	predecessorEntry: function(entry) {
		var siblings= this.siblingEntries(entry);
		if (siblings != null && siblings.length > 0) {
			var index= dojo.indexOf(siblings, entry);
			Assert.isTrue(index != -1);
			if (index > 0)
				return siblings[index - 1];
		}
		return null;
	},

	successorEntry: function(entry) {
		var siblings= this.siblingEntries(entry);
		if (siblings != null && siblings.length > 0) {
			var index= dojo.indexOf(siblings, entry);
			Assert.isTrue(index != -1);
			if (index < siblings.length - 1)
				return siblings[index + 1];
		}
		return null;
	},
	
	index: function(entry) {
		return entry != null ? dojo.indexOf(this.siblingEntries(entry), entry) : -1;
	},

	// ---- API: compare and test ------------------------------------------------------------------------------------------ 

	startsWith: function(entry, prefix) {
		while (entry) {
			if (entry === prefix) {
				return true;
			}
			entry= this.__readAccessor.getParent(entry);
		}
		return false;
	},

	isFirst: function(entry) {
		return entry === this.firstSiblingEntry(entry);
	},

	isLast: function(entry) {
		return entry === this.lastSiblingEntry(entry);
	},

	indexOf: function(entry, elementClass) {
		for (var result= 0; entry != null; result++) {
			if (entry.getElement() instanceof elementClass) {
				return result;
			}
			entry= this.__readAccessor.getParent(entry);
		}
		return -1;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__isIncluded: function(entry) {
		return !this.__filterFn || this.__filterFn(entry);
	},

	__filterInvisible: function(entries) {
		if (!this.__filterFn)
			return entries;
		
		var result= [];
		for (var i= 0; i < entries.length; i++) {
			if (this.__isIncluded(entries[i])) {
				result.push(entries[i]);
			}
		}
		return result;
	},

	__doGetElements: function(entries) {
		var result= [];
		for (var i= 0; i < entries.length; i++) {
			result.push(entries[i].getElement());
		}
		return result;
	},
	
	__firstEntryWithTag: function(entries, tag) {
		for (var i= 0; i < entries.length; i++) {
			var entry= entries[i];
			if (this.__isIncluded(entry) && entry.hasTag(tag)) {
				return entry;
			}
		}
		return null;
	},

	__sentinel: null // terminates this class definition
});

})();
