/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.structure.RowElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.ui.structure.GroupElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.SaveRunnable"); //$NON-NLS-1$

(function() {

var GroupElement					= com.ibm.team.apt.ui.structure.GroupElement;

dojo.declare("com.ibm.team.apt.ui.structure.RowElement", GroupElement, { //$NON-NLS-1$

	__parentId: null,
	__plan: null,
	__parentGroup: null,
	__isHeader: false,

	constructor: function(parentPlanItem, plan, parentGroups, isHeader) {
		if (parentPlanItem == null) {
			this.__parentId= null;
		} else {
			this.__parentId= parentPlanItem.getUuid();
		}
		this.__plan= plan;
		this.__parentGroup= parentGroups.length == 0 ? null : parentGroups[parentGroups.length - 1];
		this.__isHeader= isHeader;
	},

	getId: function() {
		return this.__parentId;
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;
		
		if (this.__isHeader)
			return other.__isHeader;
		
		if (other.__isHeader)
			return this.__isHeader;

		if (this.__parentId == null)
			return other.__parentId == null;
		
		return this.__parentId === other.__parentId;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	isHeader: function() {
		return this.__isHeader;
	},
	
	getColumnCount: function() {
		return 4;
	},
	
	getParent: function() {
		if (this.__parentId == null)
			return this.__plan;
		
		return this.__plan.getItem(this.__parentId);
	},
	
	getLabel: function() {
		if (this.__parentId == null)
			return ""; //$NON-NLS-1$
		
		return this.getParent().getSummary();
	},
	
	getElementKey: function() {
		var id= ""; //$NON-NLS-1$
		if (this.__parentGroup != null)
			id+= this.__parentGroup.getElementKey();
		
		if (this.__isHeader)
			return id + "HEADER"; //$NON-NLS-1$
		
		if (this.__parentId == null)
			return id + "DaE";  //$NON-NLS-1$
		
		return id + this.getParent().getElementKey();
	},
	
	getValue: function() {
		return this.getParent();
	},
	
	compareTo: function(other) {
		if (this.__isHeader && !other.__isHeader) {
			return -1;
		} else if (!this.__isHeader && other.__isHeader) {
			return 1;
		} else if (this.__isHeader && other.__isHeader) {
			return 0;
		} else if (this.__parentId == null && other.__parentId == null) {
			return -1;
		} else if (this.__parentId == null && other.__parentId != null) {
			return 1;
		}
		
		return 0;
	},
	
	canAdopt: function(planItem) {
		if (!(planItem instanceof com.ibm.team.apt.client.PlanItem))
			return false;
			
		var newParent= this.getParent();
		while (newParent instanceof com.ibm.team.apt.client.PlanItem) {
			if (planItem.getUuid() == newParent.getUuid())
				return false;
			
			newParent= newParent.getParent();
		}
		
		if (this.__parentGroup == null)
			return true;
		
		return this.__parentGroup.canAdopt(planItem);
	},
	
	adopt: function(planItem) {
		planItem.moveTo(this.getParent());
		if (this.__parentGroup != null) {
			this.__parentGroup.adopt(planItem);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

})();
